@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -18,14 +18,14 @@ class ScrollSnapDataTest : public testing::Test {
 TEST_F(ScrollSnapDataTest, StartAlignmentCalculation) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(10, 10, 200, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(10, 10, 200, 300), gfx::PointF(600, 800));
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kStart),
                     gfx::RectF(100, 150, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(0, 0), true,
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(0, 0), true,
                                                   true);
 
   EXPECT_TRUE(
@@ -39,14 +39,14 @@ TEST_F(ScrollSnapDataTest, StartAlignmentCalculation) {
 TEST_F(ScrollSnapDataTest, CenterAlignmentCalculation) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(10, 10, 200, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(10, 10, 200, 300), gfx::PointF(600, 800));
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kCenter),
                     gfx::RectF(100, 150, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(0, 0), true,
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(0, 0), true,
                                                   true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
@@ -59,14 +59,14 @@ TEST_F(ScrollSnapDataTest, CenterAlignmentCalculation) {
 TEST_F(ScrollSnapDataTest, EndAlignmentCalculation) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(10, 10, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(10, 10, 200, 200), gfx::PointF(600, 800));
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kEnd),
                     gfx::RectF(150, 200, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(0, 0), true,
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(0, 0), true,
                                                   true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
@@ -79,15 +79,15 @@ TEST_F(ScrollSnapDataTest, EndAlignmentCalculation) {
 TEST_F(ScrollSnapDataTest, UnreachableSnapPositionCalculation) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(100, 100));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(100, 100));
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kEnd, SnapAlignment::kStart),
                     gfx::RectF(200, 0, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(50, 50),
-                                                  true, true);
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(50, 50), true,
+                                                  true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
   // Aligning to start on x would lead the scroll offset larger than max, and
@@ -102,7 +102,7 @@ TEST_F(ScrollSnapDataTest, UnreachableSnapPositionCalculation) {
 TEST_F(ScrollSnapDataTest, FindsClosestSnapPositionIndependently) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
   SnapAreaData snap_x_only(
       ScrollSnapAlign(SnapAlignment::kNone, SnapAlignment::kStart),
       gfx::RectF(80, 0, 150, 150), false, ElementId(10));
@@ -116,10 +116,10 @@ TEST_F(ScrollSnapDataTest, FindsClosestSnapPositionIndependently) {
   container.AddSnapAreaData(snap_y_only);
   container.AddSnapAreaData(snap_on_both);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(100, 100),
-                                                  true, true);
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(100, 100), true,
+                                                  true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
   EXPECT_EQ(80, snap_position.x());
@@ -131,7 +131,7 @@ TEST_F(ScrollSnapDataTest, FindsClosestSnapPositionIndependently) {
 TEST_F(ScrollSnapDataTest, FindsClosestSnapPositionOnAxisValueBoth) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
   SnapAreaData snap_x_only(
       ScrollSnapAlign(SnapAlignment::kNone, SnapAlignment::kStart),
       gfx::RectF(80, 0, 150, 150), false, ElementId(10));
@@ -145,10 +145,10 @@ TEST_F(ScrollSnapDataTest, FindsClosestSnapPositionOnAxisValueBoth) {
   container.AddSnapAreaData(snap_x_only);
   container.AddSnapAreaData(snap_y_only);
   container.AddSnapAreaData(snap_on_both);
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(40, 120),
-                                                  true, true);
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(40, 120), true,
+                                                  true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
   EXPECT_EQ(50, snap_position.x());
@@ -160,7 +160,7 @@ TEST_F(ScrollSnapDataTest, FindsClosestSnapPositionOnAxisValueBoth) {
 TEST_F(ScrollSnapDataTest, DoesNotSnapOnNonScrolledAxis) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
   SnapAreaData snap_x_only(
       ScrollSnapAlign(SnapAlignment::kNone, SnapAlignment::kStart),
       gfx::RectF(80, 0, 150, 150), false, ElementId(10));
@@ -170,10 +170,10 @@ TEST_F(ScrollSnapDataTest, DoesNotSnapOnNonScrolledAxis) {
   container.AddSnapAreaData(snap_x_only);
   container.AddSnapAreaData(snap_y_only);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(100, 100),
-                                                  true, false);
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(100, 100), true,
+                                                  false);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
   EXPECT_EQ(80, snap_position.x());
@@ -185,7 +185,7 @@ TEST_F(ScrollSnapDataTest, DoesNotSnapOnNonScrolledAxis) {
 TEST_F(ScrollSnapDataTest, DoesNotSnapOnNonVisibleAreas) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
   SnapAreaData snap_x_only(
       ScrollSnapAlign(SnapAlignment::kNone, SnapAlignment::kStart),
       gfx::RectF(300, 400, 100, 100), false, ElementId(10));
@@ -195,9 +195,9 @@ TEST_F(ScrollSnapDataTest, DoesNotSnapOnNonVisibleAreas) {
 
   container.AddSnapAreaData(snap_x_only);
   container.AddSnapAreaData(snap_y_only);
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(0, 0), true,
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(0, 0), true,
                                                   true);
   EXPECT_FALSE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
@@ -207,7 +207,7 @@ TEST_F(ScrollSnapDataTest, DoesNotSnapOnNonVisibleAreas) {
 TEST_F(ScrollSnapDataTest, SnapOnClosestAxisFirstIfVisibilityConflicts) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
 
   // Both the areas are currently visible.
   // However, if we snap to them on x and y independently, none is visible after
@@ -227,9 +227,9 @@ TEST_F(ScrollSnapDataTest, SnapOnClosestAxisFirstIfVisibilityConflicts) {
   container.AddSnapAreaData(snap_y1);
   container.AddSnapAreaData(snap_y2);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(0, 0), true,
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(0, 0), true,
                                                   true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
@@ -242,17 +242,17 @@ TEST_F(ScrollSnapDataTest, SnapOnClosestAxisFirstIfVisibilityConflicts) {
 TEST_F(ScrollSnapDataTest, DoesNotSnapToPositionsOutsideProximityRange) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
-  container.set_proximity_range(gfx::ScrollOffset(50, 50));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
+  container.set_proximity_range(gfx::PointF(50, 50));
 
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kStart),
                     gfx::RectF(80, 160, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
-      SnapSelectionStrategy::CreateForEndPosition(gfx::ScrollOffset(100, 100),
-                                                  true, true);
+      SnapSelectionStrategy::CreateForEndPosition(gfx::PointF(100, 100), true,
+                                                  true);
   EXPECT_TRUE(
       container.FindSnapPosition(*strategy, &snap_position, &target_elements));
 
@@ -268,15 +268,15 @@ TEST_F(ScrollSnapDataTest, DoesNotSnapToPositionsOutsideProximityRange) {
 TEST_F(ScrollSnapDataTest, MandatoryReturnsToCurrentIfNoValidAreaForward) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(2000, 2000));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(2000, 2000));
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kStart),
                     gfx::RectF(600, 0, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
 
   std::unique_ptr<SnapSelectionStrategy> direction_strategy =
       SnapSelectionStrategy::CreateForDirection(
-          gfx::ScrollOffset(600, 0), gfx::ScrollOffset(5, 0),
+          gfx::PointF(600, 0), gfx::Vector2dF(5, 0),
           false /* use_fractional_deltas */);
   EXPECT_TRUE(container.FindSnapPosition(*direction_strategy, &snap_position,
                                          &target_elements));
@@ -290,7 +290,7 @@ TEST_F(ScrollSnapDataTest, MandatoryReturnsToCurrentIfNoValidAreaForward) {
 
   std::unique_ptr<SnapSelectionStrategy> end_direction_strategy =
       SnapSelectionStrategy::CreateForEndAndDirection(
-          gfx::ScrollOffset(600, 0), gfx::ScrollOffset(15, 15),
+          gfx::PointF(600, 0), gfx::Vector2dF(15, 15),
           false /* use_fractional_deltas */);
   EXPECT_TRUE(container.FindSnapPosition(*end_direction_strategy,
                                          &snap_position, &target_elements));
@@ -316,15 +316,15 @@ TEST_F(ScrollSnapDataTest, MandatoryReturnsToCurrentIfNoValidAreaForward) {
 TEST_F(ScrollSnapDataTest, MandatorySnapsBackwardIfNoValidAreaForward) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(2000, 2000));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(2000, 2000));
   SnapAreaData area(ScrollSnapAlign(SnapAlignment::kStart),
                     gfx::RectF(600, 0, 100, 100), false, ElementId(10));
   container.AddSnapAreaData(area);
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
 
   std::unique_ptr<SnapSelectionStrategy> direction_strategy =
       SnapSelectionStrategy::CreateForDirection(
-          gfx::ScrollOffset(650, 0), gfx::ScrollOffset(5, 0),
+          gfx::PointF(650, 0), gfx::Vector2d(5, 0),
           false /* use_fractional_deltas */);
   EXPECT_TRUE(container.FindSnapPosition(*direction_strategy, &snap_position,
                                          &target_elements));
@@ -338,7 +338,7 @@ TEST_F(ScrollSnapDataTest, MandatorySnapsBackwardIfNoValidAreaForward) {
 
   std::unique_ptr<SnapSelectionStrategy> end_direction_strategy =
       SnapSelectionStrategy::CreateForEndAndDirection(
-          gfx::ScrollOffset(650, 10), gfx::ScrollOffset(15, 15),
+          gfx::PointF(650, 10), gfx::Vector2d(15, 15),
           false /* use_fractional_deltas */);
   EXPECT_TRUE(container.FindSnapPosition(*end_direction_strategy,
                                          &snap_position, &target_elements));
@@ -364,7 +364,7 @@ TEST_F(ScrollSnapDataTest, MandatorySnapsBackwardIfNoValidAreaForward) {
 TEST_F(ScrollSnapDataTest, ShouldNotPassScrollSnapStopAlwaysElement) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(2000, 2000));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(2000, 2000));
   SnapAreaData must_snap_1(ScrollSnapAlign(SnapAlignment::kStart),
                            gfx::RectF(200, 0, 100, 100), true, ElementId(10));
   SnapAreaData must_snap_2(ScrollSnapAlign(SnapAlignment::kStart),
@@ -375,11 +375,11 @@ TEST_F(ScrollSnapDataTest, ShouldNotPassScrollSnapStopAlwaysElement) {
   container.AddSnapAreaData(must_snap_1);
   container.AddSnapAreaData(must_snap_2);
   container.AddSnapAreaData(closer_to_target);
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
 
   std::unique_ptr<SnapSelectionStrategy> end_direction_strategy =
       SnapSelectionStrategy::CreateForEndAndDirection(
-          gfx::ScrollOffset(0, 0), gfx::ScrollOffset(600, 0),
+          gfx::PointF(0, 0), gfx::Vector2d(600, 0),
           false /* use_fractional_deltas */);
 
   EXPECT_TRUE(container.FindSnapPosition(*end_direction_strategy,
@@ -397,7 +397,7 @@ TEST_F(ScrollSnapDataTest, ShouldNotPassScrollSnapStopAlwaysElement) {
 TEST_F(ScrollSnapDataTest, SnapStopAlwaysOverridesCoveringSnapArea) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 200), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 200), gfx::PointF(600, 800));
   SnapAreaData stop_area(ScrollSnapAlign(SnapAlignment::kStart),
                          gfx::RectF(100, 0, 100, 100), true, ElementId(10));
   SnapAreaData covering_area(ScrollSnapAlign(SnapAlignment::kStart),
@@ -406,10 +406,10 @@ TEST_F(ScrollSnapDataTest, SnapStopAlwaysOverridesCoveringSnapArea) {
   container.AddSnapAreaData(stop_area);
   container.AddSnapAreaData(covering_area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
       SnapSelectionStrategy::CreateForEndAndDirection(
-          gfx::ScrollOffset(0, 0), gfx::ScrollOffset(300, 0),
+          gfx::PointF(0, 0), gfx::Vector2d(300, 0),
           false /* use_fractional_deltas */);
 
   // The fling is from (0, 0) to (300, 0), and the destination would make
@@ -427,15 +427,15 @@ TEST_F(ScrollSnapDataTest, SnapStopAlwaysOverridesCoveringSnapArea) {
 TEST_F(ScrollSnapDataTest, SnapStopAlwaysInReverseDirection) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 300), gfx::PointF(600, 800));
   SnapAreaData stop_area(ScrollSnapAlign(SnapAlignment::kStart),
                          gfx::RectF(100, 0, 100, 100), true, ElementId(10));
   container.AddSnapAreaData(stop_area);
 
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> strategy =
       SnapSelectionStrategy::CreateForEndAndDirection(
-          gfx::ScrollOffset(150, 0), gfx::ScrollOffset(200, 0),
+          gfx::PointF(150, 0), gfx::Vector2d(200, 0),
           false /* use_fractional_deltas */);
 
   // The fling is from (150, 0) to (350, 0), but the snap position is in the
@@ -452,7 +452,7 @@ TEST_F(ScrollSnapDataTest, SnapStopAlwaysInReverseDirection) {
 TEST_F(ScrollSnapDataTest, SnapStopAlwaysNotInterferingWithDirectionStrategy) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 300), gfx::PointF(600, 800));
   SnapAreaData closer_area(ScrollSnapAlign(SnapAlignment::kStart),
                            gfx::RectF(100, 0, 1, 1), false, ElementId(10));
   SnapAreaData stop_area(ScrollSnapAlign(SnapAlignment::kStart),
@@ -462,12 +462,12 @@ TEST_F(ScrollSnapDataTest, SnapStopAlwaysNotInterferingWithDirectionStrategy) {
 
   // The DirectionStrategy should always choose the first snap position
   // regardless its scroll-snap-stop value.
-  gfx::ScrollOffset snap_position;
+  gfx::PointF snap_position;
   std::unique_ptr<SnapSelectionStrategy> direction_strategy =
       SnapSelectionStrategy::CreateForDirection(
-          gfx::ScrollOffset(90, 0), gfx::ScrollOffset(50, 0),
+          gfx::PointF(90, 0), gfx::Vector2d(50, 0),
           false /* use_fractional_deltas */);
-  snap_position = gfx::ScrollOffset();
+  snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*direction_strategy, &snap_position,
                                          &target_elements));
   EXPECT_EQ(100, snap_position.x());
@@ -479,7 +479,7 @@ TEST_F(ScrollSnapDataTest, SnapStopAlwaysNotInterferingWithDirectionStrategy) {
 TEST_F(ScrollSnapDataTest, SnapToOneTargetElementOnX) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 300), gfx::PointF(600, 800));
 
   SnapAreaData closer_area_x(ScrollSnapAlign(SnapAlignment::kStart),
                              gfx::RectF(100, 0, 1, 1), false, ElementId(10));
@@ -498,9 +498,9 @@ TEST_F(ScrollSnapDataTest, SnapToOneTargetElementOnX) {
   // should snap to the target for the x-axis. However, since the target is not
   // set for the y-axis, the target on the y-axis should be closer_area_y.
   std::unique_ptr<SnapSelectionStrategy> target_element_strategy =
-      SnapSelectionStrategy::CreateForTargetElement(gfx::ScrollOffset(0, 0));
+      SnapSelectionStrategy::CreateForTargetElement(gfx::PointF(0, 0));
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*target_element_strategy,
                                          &snap_position, &target_elements));
 
@@ -513,7 +513,7 @@ TEST_F(ScrollSnapDataTest, SnapToOneTargetElementOnX) {
 TEST_F(ScrollSnapDataTest, SnapToOneTargetElementOnY) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 200, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 200, 300), gfx::PointF(600, 800));
 
   SnapAreaData closer_area_y(ScrollSnapAlign(SnapAlignment::kStart),
                              gfx::RectF(0, 100, 1, 1), false, ElementId(10));
@@ -532,9 +532,9 @@ TEST_F(ScrollSnapDataTest, SnapToOneTargetElementOnY) {
   // should snap to the target for the y-axis. However, since the target is not
   // set for the x-axis, the target on the x-axis should be closer_area_x.
   std::unique_ptr<SnapSelectionStrategy> target_element_strategy =
-      SnapSelectionStrategy::CreateForTargetElement(gfx::ScrollOffset(0, 0));
+      SnapSelectionStrategy::CreateForTargetElement(gfx::PointF(0, 0));
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*target_element_strategy,
                                          &snap_position, &target_elements));
 
@@ -547,7 +547,7 @@ TEST_F(ScrollSnapDataTest, SnapToOneTargetElementOnY) {
 TEST_F(ScrollSnapDataTest, SnapToTwoTargetElementsMutualVisible) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 300, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 300, 300), gfx::PointF(600, 800));
 
   SnapAreaData target_area_x(ScrollSnapAlign(SnapAlignment::kStart),
                              gfx::RectF(100, 200, 1, 1), false, ElementId(10));
@@ -565,9 +565,9 @@ TEST_F(ScrollSnapDataTest, SnapToTwoTargetElementsMutualVisible) {
   // The container should snap to both target areas since they are mutually
   // visible, while ignoring the snap area that is closest to the scroll offset.
   std::unique_ptr<SnapSelectionStrategy> target_element_strategy =
-      SnapSelectionStrategy::CreateForTargetElement(gfx::ScrollOffset(0, 0));
+      SnapSelectionStrategy::CreateForTargetElement(gfx::PointF(0, 0));
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*target_element_strategy,
                                          &snap_position, &target_elements));
 
@@ -580,7 +580,7 @@ TEST_F(ScrollSnapDataTest, SnapToTwoTargetElementsMutualVisible) {
 TEST_F(ScrollSnapDataTest, SnapToTwoTargetElementsNotMutualVisible) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 300, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 300, 300), gfx::PointF(600, 800));
 
   SnapAreaData target_area_x(ScrollSnapAlign(SnapAlignment::kStart),
                              gfx::RectF(100, 500, 1, 1), false, ElementId(10));
@@ -600,9 +600,9 @@ TEST_F(ScrollSnapDataTest, SnapToTwoTargetElementsNotMutualVisible) {
   // closer to the scroll offset, and then snap to the closest mutually visible
   // snap area on the other axis.
   std::unique_ptr<SnapSelectionStrategy> target_element_strategy =
-      SnapSelectionStrategy::CreateForTargetElement(gfx::ScrollOffset(10, 0));
+      SnapSelectionStrategy::CreateForTargetElement(gfx::PointF(10, 0));
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*target_element_strategy,
                                          &snap_position, &target_elements));
 
@@ -615,7 +615,7 @@ TEST_F(ScrollSnapDataTest, SnapToTwoTargetElementsNotMutualVisible) {
 TEST_F(ScrollSnapDataTest, SnapToFocusedElementHorizontal) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kX, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 300, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 300, 300), gfx::PointF(600, 800));
   SnapAreaData snapped_area(ScrollSnapAlign(SnapAlignment::kStart),
                             gfx::RectF(0, 0, 100, 100), false, ElementId(10));
   SnapAreaData focused_area(ScrollSnapAlign(SnapAlignment::kStart),
@@ -624,11 +624,11 @@ TEST_F(ScrollSnapDataTest, SnapToFocusedElementHorizontal) {
   container.AddSnapAreaData(focused_area);
 
   // Initially both snap areas are horizontally aligned with the snap position.
-  gfx::ScrollOffset origin(0, 0);
+  gfx::PointF origin(0, 0);
   std::unique_ptr<SnapSelectionStrategy> strategy =
       SnapSelectionStrategy::CreateForTargetElement(origin);
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*strategy, &snap_position,
                                          &target_elements, ElementId(20)));
   EXPECT_EQ(0, snap_position.x());
@@ -647,7 +647,7 @@ TEST_F(ScrollSnapDataTest, SnapToFocusedElementHorizontal) {
 TEST_F(ScrollSnapDataTest, SnapToFocusedElementVertical) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kY, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 300, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 300, 300), gfx::PointF(600, 800));
   SnapAreaData snapped_area(ScrollSnapAlign(SnapAlignment::kStart),
                             gfx::RectF(0, 0, 100, 100), false, ElementId(10));
   SnapAreaData focused_area(ScrollSnapAlign(SnapAlignment::kStart),
@@ -656,11 +656,11 @@ TEST_F(ScrollSnapDataTest, SnapToFocusedElementVertical) {
   container.AddSnapAreaData(focused_area);
 
   // Initially both snap areas are vertically aligned with the snap position.
-  gfx::ScrollOffset origin(0, 0);
+  gfx::PointF origin(0, 0);
   std::unique_ptr<SnapSelectionStrategy> strategy =
       SnapSelectionStrategy::CreateForTargetElement(origin);
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*strategy, &snap_position,
                                          &target_elements, ElementId(20)));
   EXPECT_EQ(0, snap_position.y());
@@ -679,7 +679,7 @@ TEST_F(ScrollSnapDataTest, SnapToFocusedElementVertical) {
 TEST_F(ScrollSnapDataTest, SnapToFocusedElementBoth) {
   SnapContainerData container(
       ScrollSnapType(false, SnapAxis::kBoth, SnapStrictness::kMandatory),
-      gfx::RectF(0, 0, 300, 300), gfx::ScrollOffset(600, 800));
+      gfx::RectF(0, 0, 300, 300), gfx::PointF(600, 800));
   SnapAreaData snapped_area(ScrollSnapAlign(SnapAlignment::kStart),
                             gfx::RectF(0, 0, 100, 100), false, ElementId(10));
   SnapAreaData focused_area(ScrollSnapAlign(SnapAlignment::kStart),
@@ -688,11 +688,11 @@ TEST_F(ScrollSnapDataTest, SnapToFocusedElementBoth) {
   container.AddSnapAreaData(focused_area);
 
   // Initially both snap areas are coincident with the snap position.
-  gfx::ScrollOffset origin(0, 0);
+  gfx::PointF origin(0, 0);
   std::unique_ptr<SnapSelectionStrategy> strategy =
       SnapSelectionStrategy::CreateForTargetElement(origin);
 
-  gfx::ScrollOffset snap_position = gfx::ScrollOffset();
+  gfx::PointF snap_position = gfx::PointF();
   EXPECT_TRUE(container.FindSnapPosition(*strategy, &snap_position,
                                          &target_elements, ElementId(20)));
   EXPECT_EQ(0, snap_position.x());
