@@ -1,16 +1,15 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <memory>
 #include <string>
 
-#include "base/bind.h"
-#include "base/callback.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/metrics/field_trial.h"
 #include "base/run_loop.h"
-#include "base/stl_util.h"
 #include "chrome/browser/content_settings/host_content_settings_map_factory.h"
-#include "chrome/browser/media/webrtc/camera_pan_tilt_zoom_permission_context.h"
 #include "chrome/browser/media/webrtc/media_capture_devices_dispatcher.h"
 #include "chrome/browser/media/webrtc/media_stream_capture_indicator.h"
 #include "chrome/browser/media/webrtc/media_stream_device_permissions.h"
@@ -19,12 +18,14 @@
 #include "chrome/browser/permissions/permission_manager_factory.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
+#include "chrome/browser/ui/tabs/tab_enums.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/webui_url_constants.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/content_settings/browser/page_specific_content_settings.h"
 #include "components/content_settings/core/browser/host_content_settings_map.h"
+#include "components/permissions/contexts/camera_pan_tilt_zoom_permission_context.h"
 #include "components/permissions/permission_context_base.h"
 #include "components/permissions/permission_manager.h"
 #include "components/permissions/permission_request.h"
@@ -41,6 +42,7 @@
 #include "extensions/common/constants.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/blink/public/common/mediastream/media_stream_request.h"
+#include "third_party/blink/public/mojom/mediastream/media_stream.mojom.h"
 
 using content_settings::PageSpecificContentSettings;
 
@@ -53,11 +55,14 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
             blink::mojom::MediaStreamRequestResult::NUM_MEDIA_REQUEST_RESULTS) {
   }
 
-  void OnMediaStreamResponse(const blink::MediaStreamDevices& devices,
-                             blink::mojom::MediaStreamRequestResult result,
-                             std::unique_ptr<content::MediaStreamUI> ui) {
-    EXPECT_EQ(devices.empty(), !ui);
-    media_stream_devices_ = devices;
+  void OnMediaStreamResponse(
+      const blink::mojom::StreamDevicesSet& stream_devices_set,
+      blink::mojom::MediaStreamRequestResult result,
+      std::unique_ptr<content::MediaStreamUI> ui) {
+    blink::MediaStreamDevices devices_list =
+        blink::ToMediaStreamDevicesList(stream_devices_set);
+    EXPECT_EQ(devices_list.empty(), !ui);
+    media_stream_devices_ = devices_list;
     media_stream_result_ = result;
     std::move(quit_closure_).Run();
   }
@@ -70,7 +75,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
 
   PageSpecificContentSettings* GetContentSettings() {
     return PageSpecificContentSettings::GetForFrame(
-        GetWebContents()->GetMainFrame());
+        GetWebContents()->GetPrimaryMainFrame());
   }
 
   const std::string& example_audio_id() const { return example_audio_id_; }
@@ -98,7 +103,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
   void SetDevicePolicy(DeviceType device_type, Access access) {
     PrefService* prefs = Profile::FromBrowserContext(
         GetWebContents()->GetBrowserContext())->GetPrefs();
-    const char* policy_name = NULL;
+    const char* policy_name = nullptr;
     switch (device_type) {
       case DEVICE_TYPE_AUDIO:
         policy_name = prefs::kAudioCaptureAllowed;
@@ -156,14 +161,25 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
     blink::mojom::MediaStreamType video_type =
         video_id.empty() ? blink::mojom::MediaStreamType::NO_SERVICE
                          : blink::mojom::MediaStreamType::DEVICE_VIDEO_CAPTURE;
-    EXPECT_EQ(example_url(),
-              GetWebContents()->GetMainFrame()->GetLastCommittedURL());
+    if (!GetWebContents()
+             ->GetPrimaryMainFrame()
+             ->GetLastCommittedOrigin()
+             .GetURL()
+             .is_empty()) {
+      EXPECT_EQ(example_url().DeprecatedGetOriginAsURL(),
+                GetWebContents()
+                    ->GetPrimaryMainFrame()
+                    ->GetLastCommittedOrigin()
+                    .GetURL());
+    }
     int render_process_id =
-        GetWebContents()->GetMainFrame()->GetProcess()->GetID();
-    int render_frame_id = GetWebContents()->GetMainFrame()->GetRoutingID();
+        GetWebContents()->GetPrimaryMainFrame()->GetProcess()->GetID();
+    int render_frame_id =
+        GetWebContents()->GetPrimaryMainFrame()->GetRoutingID();
     return content::MediaStreamRequest(
-        render_process_id, render_frame_id, 0, example_url(), false,
-        request_type, audio_id, video_id, audio_type, video_type,
+        render_process_id, render_frame_id, 0,
+        example_url().DeprecatedGetOriginAsURL(), false, request_type, audio_id,
+        video_id, audio_type, video_type,
         /*disable_local_echo=*/false, request_pan_tilt_zoom_permission);
   }
 
@@ -179,7 +195,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
   void InitWithUrl(const GURL& url) {
     DCHECK(example_url_.is_empty());
     example_url_ = url;
-    ui_test_utils::NavigateToURL(browser(), example_url_);
+    ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), example_url_));
     EXPECT_EQ(PageSpecificContentSettings::MICROPHONE_CAMERA_NOT_ACCESSED,
               GetContentSettings()->GetMicrophoneCameraState());
   }
@@ -215,8 +231,8 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
     permissions::PermissionRequestManager* manager =
         permissions::PermissionRequestManager::FromWebContents(
             browser()->tab_strip_model()->GetActiveWebContents());
-    prompt_factory_.reset(
-        new permissions::MockPermissionPromptFactory(manager));
+    prompt_factory_ =
+        std::make_unique<permissions::MockPermissionPromptFactory>(manager);
 
     // Cleanup.
     media_stream_devices_.clear();
@@ -235,7 +251,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
     blink::MediaStreamDevice fake_video_device(
         blink::mojom::MediaStreamType::DEVICE_VIDEO_CAPTURE, example_video_id_,
         "Fake Video Device", GetControlSupport(),
-        media::MEDIA_VIDEO_FACING_NONE, base::nullopt);
+        media::MEDIA_VIDEO_FACING_NONE, absl::nullopt);
     video_devices.push_back(fake_video_device);
     MediaCaptureDevicesDispatcher::GetInstance()->SetTestVideoCaptureDevices(
         video_devices);
@@ -600,17 +616,17 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
             GetContentSettings()->GetMicrophoneCameraState());
 
   // Simulate that an a video stream is now being captured.
-  blink::MediaStreamDevices video_devices(1);
-  video_devices[0] = blink::MediaStreamDevice(
+  blink::mojom::StreamDevices devices;
+  devices.video_device = blink::MediaStreamDevice(
       blink::mojom::MediaStreamType::DEVICE_VIDEO_CAPTURE, example_video_id(),
       example_video_id());
   MediaCaptureDevicesDispatcher* dispatcher =
       MediaCaptureDevicesDispatcher::GetInstance();
-  dispatcher->SetTestVideoCaptureDevices(video_devices);
+  dispatcher->SetTestVideoCaptureDevices({devices.video_device.value()});
   std::unique_ptr<content::MediaStreamUI> video_stream_ui =
       dispatcher->GetMediaStreamCaptureIndicator()->RegisterMediaStream(
-          GetWebContents(), video_devices);
-  video_stream_ui->OnStarted(base::OnceClosure(),
+          GetWebContents(), devices);
+  video_stream_ui->OnStarted(base::RepeatingClosure(),
                              content::MediaStreamUI::SourceCallback(),
                              /*label=*/std::string(), /*screen_capture_ids=*/{},
                              content::MediaStreamUI::StateChangeCallback());
@@ -779,13 +795,14 @@ IN_PROC_BROWSER_TEST_P(MediaStreamDevicesControllerPtzTest, ContentSettings) {
   };
 
   // Prevent automatic camera permission change when camera PTZ gets updated.
-  CameraPanTiltZoomPermissionContext* camera_pan_tilt_zoom_permission_context =
-      static_cast<CameraPanTiltZoomPermissionContext*>(
-          PermissionManagerFactory::GetForProfile(
-              Profile::FromBrowserContext(
-                  GetWebContents()->GetBrowserContext()))
-              ->GetPermissionContextForTesting(
-                  ContentSettingsType::CAMERA_PAN_TILT_ZOOM));
+  permissions::CameraPanTiltZoomPermissionContext*
+      camera_pan_tilt_zoom_permission_context =
+          static_cast<permissions::CameraPanTiltZoomPermissionContext*>(
+              PermissionManagerFactory::GetForProfile(
+                  Profile::FromBrowserContext(
+                      GetWebContents()->GetBrowserContext()))
+                  ->GetPermissionContextForTesting(
+                      ContentSettingsType::CAMERA_PAN_TILT_ZOOM));
   HostContentSettingsMap* content_settings =
       HostContentSettingsMapFactory::GetForProfile(
           Profile::FromBrowserContext(GetWebContents()->GetBrowserContext()));
@@ -946,21 +963,23 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
   permission_bubble_media_access_handler_->HandleRequest(
       prompt_contents,
       CreateRequest(example_audio_id(), example_video_id(), false),
-      base::BindOnce([](const blink::MediaStreamDevices& devices,
-                        blink::mojom::MediaStreamRequestResult result,
-                        std::unique_ptr<content::MediaStreamUI> ui) {
-        // The permission may be dismissed before we have a chance to delete the
-        // request.
-        EXPECT_EQ(blink::mojom::MediaStreamRequestResult::PERMISSION_DISMISSED,
-                  result);
-      }),
+      base::BindOnce(
+          [](const blink::mojom::StreamDevicesSet& stream_devices_set,
+             blink::mojom::MediaStreamRequestResult result,
+             std::unique_ptr<content::MediaStreamUI> ui) {
+            // The permission may be dismissed before we have a chance to delete
+            // the request.
+            EXPECT_EQ(
+                blink::mojom::MediaStreamRequestResult::PERMISSION_DISMISSED,
+                result);
+          }),
       nullptr);
   // Since the mock prompt factory holds a reference to the
   // PermissionRequestManager for the WebContents and uses that reference in its
   // destructor, it has to be destroyed before the tab.
   prompt_factory_.reset();
   ASSERT_TRUE(browser()->tab_strip_model()->CloseWebContentsAt(
-      prompt_contents_index, TabStripModel::CloseTypes::CLOSE_USER_GESTURE));
+      prompt_contents_index, TabCloseTypes::CLOSE_USER_GESTURE));
   base::RunLoop().RunUntilIdle();
 
   VerifyResultState(
@@ -997,7 +1016,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
 }
 
 IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
-                       RequestCamAndMicBlockedByFeaturePolicy) {
+                       RequestCamAndMicBlockedByPermissionsPolicy) {
   InitWithUrl(embedded_test_server()->GetURL("/iframe_blank.html"));
 
   // Create a cross-origin request by using localhost as the iframe origin.
@@ -1009,13 +1028,15 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
   content::NavigateIframeToURL(GetWebContents(), "test",
                                GURL(cross_origin_url));
   content::RenderFrameHost* child_frame =
-      ChildFrameAt(GetWebContents()->GetMainFrame(), 0);
+      ChildFrameAt(GetWebContents()->GetPrimaryMainFrame(), 0);
 
   content::MediaStreamRequest request =
       CreateRequest(example_audio_id(), example_video_id(), false);
   // Make the child frame the source of the request.
   request.render_process_id = child_frame->GetProcess()->GetID();
   request.render_frame_id = child_frame->GetRoutingID();
+  request.security_origin = child_frame->GetLastCommittedOrigin().GetURL();
+
   RequestPermissions(GetWebContents(), request);
 
   ASSERT_EQ(0, prompt_factory()->TotalRequestCount());
@@ -1027,7 +1048,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
 }
 
 IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
-                       RequestCamBlockedByFeaturePolicy) {
+                       RequestCamBlockedByPermissionsPolicy) {
   InitWithUrl(embedded_test_server()->GetURL("/iframe_blank.html"));
 
   // Create a cross-origin request by using localhost as the iframe origin.
@@ -1039,13 +1060,15 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
   content::NavigateIframeToURL(GetWebContents(), "test",
                                GURL(cross_origin_url));
   content::RenderFrameHost* child_frame =
-      ChildFrameAt(GetWebContents()->GetMainFrame(), 0);
+      ChildFrameAt(GetWebContents()->GetPrimaryMainFrame(), 0);
 
   content::MediaStreamRequest request =
       CreateRequest(std::string(), example_video_id(), false);
   // Make the child frame the source of the request.
   request.render_process_id = child_frame->GetProcess()->GetID();
   request.render_frame_id = child_frame->GetRoutingID();
+  request.security_origin = child_frame->GetLastCommittedOrigin().GetURL();
+
   RequestPermissions(GetWebContents(), request);
 
   ASSERT_EQ(0, prompt_factory()->TotalRequestCount());
