@@ -1,53 +1,58 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import './icons.js';
-import 'chrome://resources/cr_elements/cr_button/cr_button.m.js';
-import 'chrome://resources/cr_elements/cr_icon_button/cr_icon_button.m.js';
-import 'chrome://resources/cr_elements/cr_icons_css.m.js';
-import 'chrome://resources/cr_elements/hidden_style_css.m.js';
-import 'chrome://resources/cr_elements/icons.m.js';
-import 'chrome://resources/cr_elements/shared_vars_css.m.js';
+import './icons.html.js';
+import 'chrome://resources/cr_elements/cr_button/cr_button.js';
+import 'chrome://resources/cr_elements/cr_icon_button/cr_icon_button.js';
+import 'chrome://resources/cr_elements/cr_icons.css.js';
+import 'chrome://resources/cr_elements/cr_hidden_style.css.js';
+import 'chrome://resources/cr_elements/icons.html.js';
+import 'chrome://resources/cr_elements/cr_shared_vars.css.js';
 import 'chrome://resources/js/action_link.js';
-import 'chrome://resources/cr_elements/action_link_css.m.js';
+import 'chrome://resources/cr_elements/action_link.css.js';
 import './strings.m.js';
 import 'chrome://resources/polymer/v3_0/iron-icon/iron-icon.js';
 import 'chrome://resources/polymer/v3_0/paper-progress/paper-progress.js';
 import 'chrome://resources/polymer/v3_0/paper-styles/color.js';
 
 import {getToastManager} from 'chrome://resources/cr_elements/cr_toast/cr_toast_manager.js';
-import {assert} from 'chrome://resources/js/assert.m.js';
-import {FocusRowBehavior} from 'chrome://resources/js/cr/ui/focus_row_behavior.m.js';
-import {focusWithoutInk} from 'chrome://resources/js/cr/ui/focus_without_ink.m.js';
-import {loadTimeData} from 'chrome://resources/js/load_time_data.m.js';
-import {HTMLEscape} from 'chrome://resources/js/util.m.js';
-import {beforeNextRender, html, mixinBehaviors, PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
+import {assert} from 'chrome://resources/js/assert_ts.js';
+import {FocusRowMixin} from 'chrome://resources/js/focus_row_mixin.js';
+import {focusWithoutInk} from 'chrome://resources/js/focus_without_ink.js';
+import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
+import {sanitizeInnerHtml} from 'chrome://resources/js/parse_html_subset.js';
+import {htmlEscape} from 'chrome://resources/js/util_ts.js';
+import {beforeNextRender, PolymerElement} from 'chrome://resources/polymer/v3_0/polymer/polymer_bundled.min.js';
 
 import {BrowserProxy} from './browser_proxy.js';
 import {DangerType, States} from './constants.js';
 import {MojomData} from './data.js';
 import {PageHandlerInterface} from './downloads.mojom-webui.js';
-import {IconLoader} from './icon_loader.js';
+import {IconLoaderImpl} from './icon_loader.js';
+import {getTemplate} from './item.html.js';
 
 export interface DownloadsItemElement {
   $: {
     'controlled-by': HTMLElement,
     'file-icon': HTMLImageElement,
-    'url': HTMLAnchorElement,
+    'file-link': HTMLAnchorElement,
     'remove': HTMLElement,
+    'url': HTMLAnchorElement,
   };
 }
 
-const DownloadsItemElementBase =
-    mixinBehaviors([FocusRowBehavior], PolymerElement) as
-    {new (): PolymerElement & FocusRowBehavior};
+const DownloadsItemElementBase = FocusRowMixin(PolymerElement);
 
 export class DownloadsItemElement extends DownloadsItemElementBase {
   static get is() {
     return 'downloads-item';
   }
 
+  static get template() {
+    return getTemplate();
+  }
+
   static get properties() {
     return {
       data: Object,
@@ -96,6 +101,12 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
         value: false,
       },
 
+      isReviewable_: {
+        computed: 'computeIsReviewable_(data.isReviewable)',
+        type: Boolean,
+        value: false,
+      },
+
       isInProgress_: {
         computed: 'computeIsInProgress_(data.state)',
         type: Boolean,
@@ -126,6 +137,12 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
         value: false,
       },
 
+      showDeepScan_: {
+        computed: 'computeShowDeepScan_(data.state)',
+        type: Boolean,
+        value: false,
+      },
+
       useFileIcon_: Boolean,
     };
   }
@@ -151,17 +168,17 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
   private showProgress_: boolean;
   private useFileIcon_: boolean;
   private restoreFocusAfterCancel_: boolean = false;
-  overrideCustomEquivalent: boolean;
+  override overrideCustomEquivalent: boolean;
 
   constructor() {
     super();
 
-    /** Used by FocusRowBehavior. */
+    /** Used by FocusRowMixin. */
     this.overrideCustomEquivalent = true;
   }
 
   /** @override */
-  ready() {
+  override ready() {
     super.ready();
 
     this.setAttribute('role', 'row');
@@ -172,8 +189,8 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
     focusWithoutInk(this.$.remove);
   }
 
-  /** Overrides FocusRowBehavior. */
-  getCustomEquivalent(sampleElement: HTMLElement): HTMLElement|null {
+  /** Overrides FocusRowMixin. */
+  override getCustomEquivalent(sampleElement: HTMLElement): HTMLElement|null {
     if (sampleElement.getAttribute('focus-type') === 'cancel') {
       return this.shadowRoot!.querySelector('[focus-type="retry"]');
     }
@@ -229,7 +246,7 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
 
     const url = `chrome://extensions/?id=${this.data.byExtId}`;
     const name = this.data.byExtName;
-    return loadTimeData.getStringF('controlledByUrl', url, HTMLEscape(name));
+    return loadTimeData.getStringF('controlledByUrl', url, htmlEscape(name));
   }
 
   private computeControlRemoveFromListAriaLabel_(): string {
@@ -242,7 +259,7 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
     if (this.data.hideDate) {
       return '';
     }
-    return assert(this.data.sinceString || this.data.dateString);
+    return this.data.sinceString || this.data.dateString;
   }
 
   private computeDescriptionVisible_(): boolean {
@@ -262,11 +279,10 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
         }
         break;
 
-      case States.MIXED_CONTENT:
-        return loadTimeData.getString('mixedContentDownloadDesc');
+      case States.INSECURE:
+        return loadTimeData.getString('insecureDownloadDesc');
 
       case States.DANGEROUS:
-        const fileName = data.fileName;
         switch (data.dangerType) {
           case DangerType.DANGEROUS_FILE:
             return loadTimeData.getString('dangerFileDesc');
@@ -292,7 +308,8 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
 
       case States.ASYNC_SCANNING:
         return loadTimeData.getString('asyncScanningDownloadDesc');
-
+      case States.PROMPT_FOR_SCANNING:
+        return loadTimeData.getString('promptForScanningDesc');
       case States.IN_PROGRESS:
       case States.PAUSED:  // Fallthrough.
         return data.progressStatusText;
@@ -332,6 +349,10 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
       if (this.data.state === States.ASYNC_SCANNING) {
         return 'cr:info';
       }
+
+      if (this.data.state === States.PROMPT_FOR_SCANNING) {
+        return 'cr:warning';
+      }
     }
     if (this.isDangerous_) {
       return 'cr:error';
@@ -363,6 +384,10 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
       if (this.data.state === States.ASYNC_SCANNING) {
         return 'grey';
       }
+
+      if (this.data.state === States.PROMPT_FOR_SCANNING) {
+        return 'yellow';
+      }
     }
     if (this.isDangerous_) {
       return 'red';
@@ -381,7 +406,7 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
 
   private computeIsDangerous_(): boolean {
     return this.data.state === States.DANGEROUS ||
-        this.data.state === States.MIXED_CONTENT;
+        this.data.state === States.INSECURE;
   }
 
   private computeIsInProgress_(): boolean {
@@ -397,6 +422,10 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
          this.data.dangerType === DangerType.DANGEROUS_ACCOUNT_COMPROMISE);
   }
 
+  private computeIsReviewable_(): boolean {
+    return this.data.isReviewable;
+  }
+
   private toggleButtonClass_() {
     this.shadowRoot!.querySelector('#pauseOrResume')!.classList.toggle(
         'action-button',
@@ -441,7 +470,8 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
 
   private computeShowProgress_(): boolean {
     return this.showCancel_ && this.data.percent >= -1 &&
-        this.data.state !== States.ASYNC_SCANNING;
+        this.data.state !== States.ASYNC_SCANNING &&
+        this.data.state !== States.PROMPT_FOR_SCANNING;
   }
 
   private computeShowOpenNow_(): boolean {
@@ -449,6 +479,10 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
     return this.data.state === States.ASYNC_SCANNING && allowOpenNow;
   }
 
+  private computeShowDeepScan_(): boolean {
+    return this.data.state === States.PROMPT_FOR_SCANNING;
+  }
+
   private computeTag_(): string {
     switch (this.data.state) {
       case States.CANCELLED:
@@ -471,7 +505,7 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
   }
 
   private observeControlledBy_() {
-    this.$['controlled-by'].innerHTML = this.controlledBy_;
+    this.$['controlled-by'].innerHTML = sanitizeInnerHtml(this.controlledBy_);
     if (this.controlledBy_) {
       const link = this.shadowRoot!.querySelector('#controlled-by a');
       link!.setAttribute('focus-row-control', '');
@@ -498,10 +532,12 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
       this.useFileIcon_ = false;
     } else if (this.data.state === States.ASYNC_SCANNING) {
       this.useFileIcon_ = false;
+    } else if (this.data.state === States.PROMPT_FOR_SCANNING) {
+      this.useFileIcon_ = false;
     } else {
-      this.$.url.href = assert(this.data.url);
+      this.$.url.href = this.data.url;
       const path = this.data.filePath;
-      IconLoader.getInstance()
+      IconLoaderImpl.getInstance()
           .loadIcon(this.$['file-icon'], path)
           .then(success => {
             if (path === this.data.filePath &&
@@ -525,6 +561,18 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
     this.mojoHandler_!.openDuringScanningRequiringGesture(this.data.id);
   }
 
+  private onDeepScanTap_() {
+    this.mojoHandler_!.deepScan(this.data.id);
+  }
+
+  private onBypassDeepScanTap_() {
+    this.mojoHandler_!.bypassDeepScanRequiringGesture(this.data.id);
+  }
+
+  private onReviewDangerousTap_() {
+    this.mojoHandler_!.reviewDangerousRequiringGesture(this.data.id);
+  }
+
   private onDragStart_(e: Event) {
     e.preventDefault();
     this.mojoHandler_!.drag(this.data.id);
@@ -559,7 +607,7 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
       // Make the file name collapsible.
       p.collapsible = !!p.arg;
     });
-    const canUndo = !this.data.isDangerous && !this.data.isMixedContent;
+    const canUndo = !this.data.isDangerous && !this.data.isInsecure;
     getToastManager().showForStringPieces(pieces, /* hideSlotted= */ !canUndo);
 
     // Stop propagating a click to the document to remove toast.
@@ -591,9 +639,11 @@ export class DownloadsItemElement extends DownloadsItemElementBase {
       }
     });
   }
+}
 
-  static get template() {
-    return html`{__html_template__}`;
+declare global {
+  interface HTMLElementTagNameMap {
+    'downloads-item': DownloadsItemElement;
   }
 }
 
