@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,6 @@
 
 #include <string>
 
-#include "base/strings/string16.h"
 
 namespace version_utils {
 namespace win {
@@ -19,7 +18,7 @@ std::string GetFullWindowsVersion();
 // Return a formatted version of the update cohort string
 // IDS_VERSION_UI_COHORT_NAME filled with the update cohort of this Chromium
 // install.
-base::string16 GetCohortVersionInfo();
+std::u16string GetCohortVersionInfo();
 
 }  // namespace win
 }  // namespace version_utils
