@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -30,74 +30,6 @@
 
 namespace chrome_common {
 
-#if BUILDFLAG(ENABLE_PLUGINS)
-TEST(ChromeContentClientTest, FindMostRecent) {
-  std::vector<std::unique_ptr<content::PepperPluginInfo>> version_vector;
-  // Test an empty vector.
-  EXPECT_EQ(nullptr, ChromeContentClient::FindMostRecentPlugin(version_vector));
-
-  // Now test the vector with one element.
-  content::PepperPluginInfo info;
-  info.version = "1.0.0.0";
-  version_vector.push_back(std::make_unique<content::PepperPluginInfo>(info));
-
-  content::PepperPluginInfo* most_recent =
-      ChromeContentClient::FindMostRecentPlugin(version_vector);
-  EXPECT_EQ("1.0.0.0", most_recent->version);
-
-  content::PepperPluginInfo info5;
-  info5.version = "5.0.12.1";
-  content::PepperPluginInfo info6_12;
-  info6_12.version = "6.0.0.12";
-  content::PepperPluginInfo info6_13;
-  info6_13.version = "6.0.0.13";
-
-  // Test highest version is picked.
-  version_vector.clear();
-  version_vector.push_back(std::make_unique<content::PepperPluginInfo>(info5));
-  version_vector.push_back(
-      std::make_unique<content::PepperPluginInfo>(info6_12));
-  version_vector.push_back(
-      std::make_unique<content::PepperPluginInfo>(info6_13));
-
-  most_recent = ChromeContentClient::FindMostRecentPlugin(version_vector);
-  EXPECT_EQ("6.0.0.13", most_recent->version);
-
-  // Test that order does not matter, validates tests below.
-  version_vector.clear();
-  version_vector.push_back(
-      std::make_unique<content::PepperPluginInfo>(info6_13));
-  version_vector.push_back(
-      std::make_unique<content::PepperPluginInfo>(info6_12));
-  version_vector.push_back(std::make_unique<content::PepperPluginInfo>(info5));
-
-  most_recent = ChromeContentClient::FindMostRecentPlugin(version_vector);
-  EXPECT_EQ("6.0.0.13", most_recent->version);
-
-  // Test real scenarios.
-  content::PepperPluginInfo component_flash;
-  component_flash.version = "4.3.2.1";
-  component_flash.is_external = false;
-  component_flash.name = "component_flash";
-
-  content::PepperPluginInfo system_flash;
-  system_flash.version = "4.3.2.1";
-  system_flash.is_external = true;
-  system_flash.name = "system_flash";
-
-  // The order here should be:
-  // 1. System Flash.
-  // 2. Component update.
-  version_vector.clear();
-  version_vector.push_back(
-      std::make_unique<content::PepperPluginInfo>(system_flash));
-  version_vector.push_back(
-      std::make_unique<content::PepperPluginInfo>(component_flash));
-  most_recent = ChromeContentClient::FindMostRecentPlugin(version_vector);
-  EXPECT_STREQ("system_flash", most_recent->name.c_str());
-}
-#endif  // BUILDFLAG(ENABLE_PLUGINS)
-
 TEST(ChromeContentClientTest, AdditionalSchemes) {
 #if BUILDFLAG(ENABLE_EXTENSIONS)
   EXPECT_TRUE(url::IsStandard(
@@ -122,6 +54,7 @@ TEST(ChromeContentClientTest, AdditionalSchemes) {
     "chrome://foo/",
     "chrome-untrusted://foo/",
     "chrome-search://foo/",
+    "isolated-app://foo/",
 #if BUILDFLAG(ENABLE_EXTENSIONS)
     "chrome-extension://foo/",
 #endif
@@ -136,7 +69,7 @@ TEST(ChromeContentClientTest, AdditionalSchemes) {
 
   GURL chrome_url(content::GetWebUIURL("dummyurl"));
   EXPECT_TRUE(network::IsUrlPotentiallyTrustworthy(chrome_url));
-  EXPECT_FALSE(content::OriginCanAccessServiceWorkers(chrome_url));
+  EXPECT_TRUE(content::OriginCanAccessServiceWorkers(chrome_url));
   EXPECT_TRUE(
       network::IsOriginPotentiallyTrustworthy(url::Origin::Create(chrome_url)));
 }
