@@ -1,4 +1,4 @@
-// Copyright 2021 The Chromium Authors. All rights reserved.
+// Copyright 2021 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,6 +13,15 @@ namespace prefs {
 COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
 extern const char kSecondaryGoogleAccountSigninAllowed[];
 
+COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
+extern const char kAccountAppsAvailability[];
+
+COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
+extern const char kIsAvailableInArcKey[];
+
+COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
+extern const char kSecondaryGoogleAccountUsage[];
+
 }  // namespace prefs
 }  // namespace account_manager
 
