@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,6 +10,7 @@
 #include <vector>
 
 #include "base/feature_list.h"
+#include "base/logging.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/autofill/core/browser/data_model/autofill_structured_address_test_utils.h"
 #include "components/autofill/core/browser/data_model/autofill_structured_address_utils.h"
@@ -19,7 +20,6 @@
 using base::ASCIIToUTF16;
 
 namespace autofill {
-namespace structured_address {
 
 using AddressComponentTestValues = std::vector<AddressComponentTestValue>;
 
@@ -48,14 +48,14 @@ struct LastNameParserTestRecord {
 
 // Function to test the parsing of a name from the full (unstructured)
 // representation into its subcomponents.
-void TestNameParsing(const base::string16& full_with_prefix,
-                     const base::string16& honorific,
-                     const base::string16& first,
-                     const base::string16& middle,
-                     const base::string16& last,
-                     const base::string16& last_first,
-                     const base::string16& last_conjunction,
-                     const base::string16& last_second) {
+void TestNameParsing(const std::u16string& full_with_prefix,
+                     const std::u16string& honorific,
+                     const std::u16string& first,
+                     const std::u16string& middle,
+                     const std::u16string& last,
+                     const std::u16string& last_first,
+                     const std::u16string& last_conjunction,
+                     const std::u16string& last_second) {
   SCOPED_TRACE(full_with_prefix);
   NameFullWithPrefix name;
   name.SetValueForTypeIfPossible(NAME_FULL_WITH_HONORIFIC_PREFIX,
@@ -66,7 +66,7 @@ void TestNameParsing(const base::string16& full_with_prefix,
   EXPECT_EQ(name.GetValueForType(NAME_FULL_WITH_HONORIFIC_PREFIX),
             full_with_prefix);
   // TODO(crbug.com/1113617): Honorifics are temporally disabled.
-  if (structured_address::HonorificPrefixEnabled()) {
+  if (HonorificPrefixEnabled()) {
     EXPECT_EQ(name.GetValueForType(NAME_HONORIFIC_PREFIX), honorific);
   }
 
@@ -84,10 +84,10 @@ void TestNameParsing(const base::string16& full_with_prefix,
 }
 
 // Testing function for parsing a |NAME_LAST| into its subcomponents.
-void TestLastNameParsing(const base::string16& last_name,
-                         const base::string16& target_first,
-                         const base::string16& target_conjunction,
-                         const base::string16& target_second) {
+void TestLastNameParsing(const std::u16string& last_name,
+                         const std::u16string& target_first,
+                         const std::u16string& target_conjunction,
+                         const std::u16string& target_second) {
   SCOPED_TRACE(last_name);
 
   NameLast last_name_component(nullptr);
@@ -167,6 +167,10 @@ TEST(AutofillStructuredName, ParseFullName) {
       // Name with multiple middle names.
       {"George Walker Junior Bush", "", "George", "Walker Junior", "Bush", "",
        "", "Bush"},
+      // Name with a middle name initial.
+      {"George W Bush", "", "George", "W", "Bush", "", "", "Bush"},
+      // Name with a middle name initial.
+      {"George W. Bush", "", "George", "W.", "Bush", "", "", "Bush"},
       // Name with a single middle name.
       {"George Walker Bush", "", "George", "Walker", "Bush", "", "", "Bush"},
       // Name without names.
@@ -303,57 +307,44 @@ TEST(AutofillStructuredName, HasMiddleNameInitialsCharacteristics) {
 
 // Test the reduction of a name to its initials.
 TEST(AutofillStructuredName, ReduceToInitials) {
-  EXPECT_EQ(ReduceToInitials(base::ASCIIToUTF16("")), base::ASCIIToUTF16(""));
-  EXPECT_EQ(ReduceToInitials(base::ASCIIToUTF16("George")),
-            base::ASCIIToUTF16("G"));
-  EXPECT_EQ(ReduceToInitials(base::ASCIIToUTF16("George Walker")),
-            base::ASCIIToUTF16("GW"));
-  EXPECT_EQ(ReduceToInitials(base::ASCIIToUTF16("michael myers")),
-            base::ASCIIToUTF16("MM"));
-  EXPECT_EQ(ReduceToInitials(base::ASCIIToUTF16("Hans-Peter")),
-            base::ASCIIToUTF16("HP"));
+  EXPECT_EQ(ReduceToInitials(u""), u"");
+  EXPECT_EQ(ReduceToInitials(u"George"), u"G");
+  EXPECT_EQ(ReduceToInitials(u"George Walker"), u"GW");
+  EXPECT_EQ(ReduceToInitials(u"michael myers"), u"MM");
+  EXPECT_EQ(ReduceToInitials(u"Hans-Peter"), u"HP");
 }
 
 // Test getting the field type |NAME_MIDDLE_INITIAL|.
 TEST(AutofillStructuredName, GetNameMiddleInitial) {
   NameFull full_name;
 
-  full_name.SetValueForTypeIfPossible(NAME_MIDDLE,
-                                      base::ASCIIToUTF16("Michael"),
+  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, u"Michael",
                                       VerificationStatus::kObserved);
 
-  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::ASCIIToUTF16("M"));
+  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL), u"M");
 
-  full_name.SetValueForTypeIfPossible(NAME_MIDDLE,
-                                      base::ASCIIToUTF16("Michael Myers"),
+  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, u"Michael Myers",
                                       VerificationStatus::kObserved);
 
-  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::ASCIIToUTF16("MM"));
+  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL), u"MM");
 
-  full_name.SetValueForTypeIfPossible(NAME_MIDDLE,
-                                      base::ASCIIToUTF16("george walker"),
+  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, u"george walker",
                                       VerificationStatus::kObserved);
-  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::ASCIIToUTF16("GW"));
+  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL), u"GW");
 
   // The the set value already has the characteristics of initials, the value
   // should be returned as it is.
-  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16("GW"),
+  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, u"GW",
                                       VerificationStatus::kObserved);
-  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::ASCIIToUTF16("GW"));
+  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL), u"GW");
 
-  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16("G. W."),
+  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, u"G. W.",
                                       VerificationStatus::kObserved);
-  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::ASCIIToUTF16("G. W."));
+  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL), u"G. W.");
 
-  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16("G.-W."),
+  full_name.SetValueForTypeIfPossible(NAME_MIDDLE, u"G.-W.",
                                       VerificationStatus::kObserved);
-  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::ASCIIToUTF16("G.-W."));
+  EXPECT_EQ(full_name.GetValueForType(NAME_MIDDLE_INITIAL), u"G.-W.");
 }
 
 TEST(AutofillStructuredName, TestGetSupportedTypes_FullNameWithPrefix) {
@@ -380,15 +371,12 @@ TEST(AutofillStructuredName, TestGetSupportedTypes_FullName) {
 TEST(AutofillStructuredName, TestSettingMiddleNameInitial) {
   NameFullWithPrefix full_name_with_prefix;
   EXPECT_EQ(full_name_with_prefix.GetValueForType(NAME_MIDDLE),
-            base::string16());
+            std::u16string());
 
   EXPECT_TRUE(full_name_with_prefix.SetValueForTypeIfPossible(
-      NAME_MIDDLE_INITIAL, base::UTF8ToUTF16("M"),
-      VerificationStatus::kObserved));
-  EXPECT_EQ(full_name_with_prefix.GetValueForType(NAME_MIDDLE_INITIAL),
-            base::UTF8ToUTF16("M"));
-  EXPECT_EQ(full_name_with_prefix.GetValueForType(NAME_MIDDLE),
-            base::UTF8ToUTF16("M"));
+      NAME_MIDDLE_INITIAL, u"M", VerificationStatus::kObserved));
+  EXPECT_EQ(full_name_with_prefix.GetValueForType(NAME_MIDDLE_INITIAL), u"M");
+  EXPECT_EQ(full_name_with_prefix.GetValueForType(NAME_MIDDLE), u"M");
 }
 
 TEST(AutofillStructuredName, MergePermutatedNames) {
@@ -396,21 +384,20 @@ TEST(AutofillStructuredName, MergePermutatedNames) {
   NameFull two;
 
   // The first component has an observed substructure of the full name.
-  EXPECT_TRUE(one.SetValueForTypeIfPossible(
-      NAME_FIRST, base::ASCIIToUTF16("First"), VerificationStatus::kObserved));
-  EXPECT_TRUE(one.SetValueForTypeIfPossible(
-      NAME_LAST, base::ASCIIToUTF16("Last"), VerificationStatus::kObserved));
+  EXPECT_TRUE(one.SetValueForTypeIfPossible(NAME_FIRST, u"First",
+                                            VerificationStatus::kObserved));
+  EXPECT_TRUE(one.SetValueForTypeIfPossible(NAME_LAST, u"Last",
+                                            VerificationStatus::kObserved));
   one.CompleteFullTree();
 
   // The formatted full name has the canonical representation "FIRST LAST".
-  EXPECT_EQ(one.GetValueForType(NAME_FULL), base::ASCIIToUTF16("First Last"));
+  EXPECT_EQ(one.GetValueForType(NAME_FULL), u"First Last");
   EXPECT_EQ(one.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kFormatted);
 
   // In contrast, the second component has a verified name in an alternative
   // representation "LAST, FIRST"
-  EXPECT_TRUE(two.SetValueForTypeIfPossible(NAME_FULL,
-                                            base::ASCIIToUTF16("Last, First"),
+  EXPECT_TRUE(two.SetValueForTypeIfPossible(NAME_FULL, u"Last, First",
                                             VerificationStatus::kUserVerified));
   EXPECT_EQ(two.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kUserVerified);
@@ -418,20 +405,20 @@ TEST(AutofillStructuredName, MergePermutatedNames) {
   EXPECT_EQ(two.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kUserVerified);
 
-  EXPECT_EQ(two.GetValueForType(NAME_FIRST), base::ASCIIToUTF16("First"));
-  EXPECT_EQ(two.GetValueForType(NAME_LAST), base::ASCIIToUTF16("Last"));
+  EXPECT_EQ(two.GetValueForType(NAME_FIRST), u"First");
+  EXPECT_EQ(two.GetValueForType(NAME_LAST), u"Last");
 
   EXPECT_TRUE(one.MergeWithComponent(two));
 
   // It is expected that the alternative representation of the second component
   // is merged into the first one, while maintaining the observed substructure.
-  EXPECT_EQ(one.GetValueForType(NAME_FULL), base::ASCIIToUTF16("Last, First"));
+  EXPECT_EQ(one.GetValueForType(NAME_FULL), u"Last, First");
   EXPECT_EQ(one.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kUserVerified);
-  EXPECT_EQ(one.GetValueForType(NAME_FIRST), base::ASCIIToUTF16("First"));
+  EXPECT_EQ(one.GetValueForType(NAME_FIRST), u"First");
   EXPECT_EQ(one.GetVerificationStatusForType(NAME_FIRST),
             VerificationStatus::kObserved);
-  EXPECT_EQ(one.GetValueForType(NAME_LAST), base::ASCIIToUTF16("Last"));
+  EXPECT_EQ(one.GetValueForType(NAME_LAST), u"Last");
   EXPECT_EQ(one.GetVerificationStatusForType(NAME_LAST),
             VerificationStatus::kObserved);
 }
@@ -812,22 +799,17 @@ TEST(AutofillStructuredName, TestCopyConstructuror) {
   NameFull orginal;
   // The first name has an incorrect componentization of the last name, but
   // a correctly observed structure of title, first, middle, last.
-  orginal.SetValueForTypeIfPossible(
-      NAME_FULL, base::ASCIIToUTF16("Mr Pablo Diego Ruiz y Picasso"),
-      VerificationStatus::kUserVerified);
-  orginal.SetValueForTypeIfPossible(NAME_HONORIFIC_PREFIX,
-                                    base::ASCIIToUTF16("Mr"),
+  orginal.SetValueForTypeIfPossible(NAME_FULL, u"Mr Pablo Diego Ruiz y Picasso",
+                                    VerificationStatus::kUserVerified);
+  orginal.SetValueForTypeIfPossible(NAME_HONORIFIC_PREFIX, u"Mr",
                                     VerificationStatus::kObserved);
-  orginal.SetValueForTypeIfPossible(NAME_FIRST,
-                                    base::ASCIIToUTF16("Pablo Diego"),
+  orginal.SetValueForTypeIfPossible(NAME_FIRST, u"Pablo Diego",
                                     VerificationStatus::kObserved);
-  orginal.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16(""),
+  orginal.SetValueForTypeIfPossible(NAME_MIDDLE, u"",
                                     VerificationStatus::kObserved);
-  orginal.SetValueForTypeIfPossible(NAME_LAST,
-                                    base::ASCIIToUTF16("Ruiz y Picasso"),
+  orginal.SetValueForTypeIfPossible(NAME_LAST, u"Ruiz y Picasso",
                                     VerificationStatus::kObserved);
-  orginal.SetValueForTypeIfPossible(NAME_LAST_SECOND,
-                                    base::ASCIIToUTF16("Ruiz y Picasso"),
+  orginal.SetValueForTypeIfPossible(NAME_LAST_SECOND, u"Ruiz y Picasso",
                                     VerificationStatus::kParsed);
 
   NameFull copy = orginal;
@@ -837,27 +819,24 @@ TEST(AutofillStructuredName, TestCopyConstructuror) {
 TEST(AutofillStructuredName,
      MigrationFromLegacyStructure_WithFullName_Unverified) {
   NameFull name;
-  name.SetValueForTypeIfPossible(NAME_FULL,
-                                 base::ASCIIToUTF16("Thomas Neo Anderson"),
+  name.SetValueForTypeIfPossible(NAME_FULL, u"Thomas Neo Anderson",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_FIRST, base::ASCIIToUTF16("Thomas"),
+  name.SetValueForTypeIfPossible(NAME_FIRST, u"Thomas",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16("Neo"),
+  name.SetValueForTypeIfPossible(NAME_MIDDLE, u"Neo",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_LAST, base::ASCIIToUTF16("Anderson"),
+  name.SetValueForTypeIfPossible(NAME_LAST, u"Anderson",
                                  VerificationStatus::kNoStatus);
 
   name.MigrateLegacyStructure(false);
 
   // Since the full name is set and the profile is not verified it is promoted
   // to observed. All other tokens are reset.
-  EXPECT_EQ(name.GetValueForType(NAME_FULL),
-            base::ASCIIToUTF16("Thomas Neo Anderson"));
-  EXPECT_EQ(name.GetValueForType(NAME_FIRST), base::ASCIIToUTF16("Thomas"));
-  EXPECT_EQ(name.GetValueForType(NAME_MIDDLE), base::ASCIIToUTF16("Neo"));
-  EXPECT_EQ(name.GetValueForType(NAME_LAST), base::ASCIIToUTF16("Anderson"));
-  EXPECT_EQ(name.GetValueForType(NAME_LAST_SECOND),
-            base::ASCIIToUTF16("Anderson"));
+  EXPECT_EQ(name.GetValueForType(NAME_FULL), u"Thomas Neo Anderson");
+  EXPECT_EQ(name.GetValueForType(NAME_FIRST), u"Thomas");
+  EXPECT_EQ(name.GetValueForType(NAME_MIDDLE), u"Neo");
+  EXPECT_EQ(name.GetValueForType(NAME_LAST), u"Anderson");
+  EXPECT_EQ(name.GetValueForType(NAME_LAST_SECOND), u"Anderson");
 
   EXPECT_EQ(name.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kObserved);
@@ -874,27 +853,24 @@ TEST(AutofillStructuredName,
 TEST(AutofillStructuredName,
      MigrationFromLegacyStructure_WithFullName_Verified) {
   NameFull name;
-  name.SetValueForTypeIfPossible(NAME_FULL,
-                                 base::ASCIIToUTF16("Thomas Neo Anderson"),
+  name.SetValueForTypeIfPossible(NAME_FULL, u"Thomas Neo Anderson",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_FIRST, base::ASCIIToUTF16("Thomas"),
+  name.SetValueForTypeIfPossible(NAME_FIRST, u"Thomas",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16("Neo"),
+  name.SetValueForTypeIfPossible(NAME_MIDDLE, u"Neo",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_LAST, base::ASCIIToUTF16("Anderson"),
+  name.SetValueForTypeIfPossible(NAME_LAST, u"Anderson",
                                  VerificationStatus::kNoStatus);
 
   name.MigrateLegacyStructure(true);
 
   // Since the full name is set and the profile is verified, it is promoted to
   // kUserVerified. All other tokens are reset.
-  EXPECT_EQ(name.GetValueForType(NAME_FULL),
-            base::ASCIIToUTF16("Thomas Neo Anderson"));
-  EXPECT_EQ(name.GetValueForType(NAME_FIRST), base::ASCIIToUTF16("Thomas"));
-  EXPECT_EQ(name.GetValueForType(NAME_MIDDLE), base::ASCIIToUTF16("Neo"));
-  EXPECT_EQ(name.GetValueForType(NAME_LAST), base::ASCIIToUTF16("Anderson"));
-  EXPECT_EQ(name.GetValueForType(NAME_LAST_SECOND),
-            base::ASCIIToUTF16("Anderson"));
+  EXPECT_EQ(name.GetValueForType(NAME_FULL), u"Thomas Neo Anderson");
+  EXPECT_EQ(name.GetValueForType(NAME_FIRST), u"Thomas");
+  EXPECT_EQ(name.GetValueForType(NAME_MIDDLE), u"Neo");
+  EXPECT_EQ(name.GetValueForType(NAME_LAST), u"Anderson");
+  EXPECT_EQ(name.GetValueForType(NAME_LAST_SECOND), u"Anderson");
 
   EXPECT_EQ(name.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kUserVerified);
@@ -912,13 +888,12 @@ TEST(AutofillStructuredName, MigrationFromLegacyStructure_WithoutFullName) {
   NameFull name;
   // The first name has an incorrect componentization of the last name, but
   // a correctly observed structure of title, first, middle, last.
-  name.SetValueForTypeIfPossible(NAME_FULL, base::ASCIIToUTF16(""),
+  name.SetValueForTypeIfPossible(NAME_FULL, u"", VerificationStatus::kNoStatus);
+  name.SetValueForTypeIfPossible(NAME_FIRST, u"Thomas",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_FIRST, base::ASCIIToUTF16("Thomas"),
+  name.SetValueForTypeIfPossible(NAME_MIDDLE, u"Neo",
                                  VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_MIDDLE, base::ASCIIToUTF16("Neo"),
-                                 VerificationStatus::kNoStatus);
-  name.SetValueForTypeIfPossible(NAME_LAST, base::ASCIIToUTF16("Anderson"),
+  name.SetValueForTypeIfPossible(NAME_LAST, u"Anderson",
                                  VerificationStatus::kNoStatus);
 
   name.MigrateLegacyStructure(false);
@@ -927,10 +902,10 @@ TEST(AutofillStructuredName, MigrationFromLegacyStructure_WithoutFullName) {
   // This is an edge case that normally should not happen.
   // Also, it is ignored that the profile might be verified because a verified
   // profile should contain a full name (or potentially no name).
-  EXPECT_EQ(name.GetValueForType(NAME_FULL), base::ASCIIToUTF16(""));
-  EXPECT_EQ(name.GetValueForType(NAME_FIRST), base::ASCIIToUTF16("Thomas"));
-  EXPECT_EQ(name.GetValueForType(NAME_MIDDLE), base::ASCIIToUTF16("Neo"));
-  EXPECT_EQ(name.GetValueForType(NAME_LAST), base::ASCIIToUTF16("Anderson"));
+  EXPECT_EQ(name.GetValueForType(NAME_FULL), u"");
+  EXPECT_EQ(name.GetValueForType(NAME_FIRST), u"Thomas");
+  EXPECT_EQ(name.GetValueForType(NAME_MIDDLE), u"Neo");
+  EXPECT_EQ(name.GetValueForType(NAME_LAST), u"Anderson");
 
   EXPECT_EQ(name.GetVerificationStatusForType(NAME_FULL),
             VerificationStatus::kNoStatus);
@@ -1056,8 +1031,8 @@ TEST(AutofillStructuredName, MergeSubsetLastname_WithNonSpaceSeparators) {
   // After normalization, the two names should have a single-token-superset
   // relation.
   SortedTokenComparisonResult token_comparison_result =
-      CompareSortedTokens(name.ValueForComparisonForTesting(),
-                          subset_name.ValueForComparisonForTesting());
+      CompareSortedTokens(name.ValueForComparisonForTesting(subset_name),
+                          subset_name.ValueForComparisonForTesting(name));
   EXPECT_TRUE(token_comparison_result.IsSingleTokenSuperset());
 
   // Without normalization, the two names should be considered distinct.
@@ -1120,5 +1095,4 @@ TEST(AutofillStructuredName, MergeSubsetLastname2) {
   VerifyTestValues(&name, name_values);
 }
 
-}  // namespace structured_address
 }  // namespace autofill
