@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,9 +6,9 @@
 
 #include <memory>
 
-#include "base/callback.h"
-#include "base/callback_helpers.h"
-#include "base/macros.h"
+#include "base/functional/callback.h"
+#include "base/functional/callback_helpers.h"
+#include "base/memory/raw_ptr.h"
 #include "components/captive_portal/content/captive_portal_service.h"
 #include "components/captive_portal/content/captive_portal_tab_reloader.h"
 #include "components/embedder_support/pref_names.h"
@@ -58,6 +58,11 @@ class CaptivePortalTabHelperTest : public content::RenderViewHostTestHarness {
  public:
   CaptivePortalTabHelperTest()
       : mock_reloader_(new testing::StrictMock<MockCaptivePortalTabReloader>) {}
+
+  CaptivePortalTabHelperTest(const CaptivePortalTabHelperTest&) = delete;
+  CaptivePortalTabHelperTest& operator=(const CaptivePortalTabHelperTest&) =
+      delete;
+
   ~CaptivePortalTabHelperTest() override {}
 
   void SetUp() override {
@@ -119,8 +124,8 @@ class CaptivePortalTabHelperTest : public content::RenderViewHostTestHarness {
         .Times(1);
     auto navigation = content::NavigationSimulator::CreateBrowserInitiated(
         url, web_contents());
-    navigation->SetResolveErrorInfo({net::ERR_CERT_COMMON_NAME_INVALID,
-                                     true /* is_secure_network_error */});
+    navigation->SetResolveErrorInfo(net::ResolveErrorInfo(
+        net::ERR_CERT_COMMON_NAME_INVALID, true /* is_secure_network_error */));
     navigation->Fail(net::ERR_NAME_NOT_RESOLVED);
     EXPECT_CALL(mock_reloader(),
                 OnLoadCommitted(net::ERR_NAME_NOT_RESOLVED,
@@ -188,9 +193,7 @@ class CaptivePortalTabHelperTest : public content::RenderViewHostTestHarness {
   std::unique_ptr<CaptivePortalTabHelper> tab_helper_;
 
   // Owned by |tab_helper_|.
-  testing::StrictMock<MockCaptivePortalTabReloader>* mock_reloader_;
-
-  DISALLOW_COPY_AND_ASSIGN(CaptivePortalTabHelperTest);
+  raw_ptr<testing::StrictMock<MockCaptivePortalTabReloader>> mock_reloader_;
 };
 
 TEST_F(CaptivePortalTabHelperTest, HttpSuccess) {
