@@ -1,4 +1,4 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,11 +8,12 @@
 #include <memory>
 #include <tuple>
 
-#include "base/bind.h"
-#include "base/callback_helpers.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/json/values_util.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
+#include "base/memory/raw_ptr.h"
 #include "base/no_destructor.h"
 #include "base/strings/string_util.h"
 #include "build/build_config.h"
@@ -22,12 +23,13 @@
 #include "components/user_prefs/user_prefs.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/navigation_handle.h"
+#include "content/public/browser/web_contents.h"
 #include "media/base/media_drm_key_type.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 #include "url/origin.h"
 #include "url/url_constants.h"
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 #include "base/task/task_traits.h"
 #include "base/task/thread_pool.h"
 #include "base/threading/scoped_blocking_call.h"
@@ -79,17 +81,15 @@ const char kMimeType[] = "mime_type";
 const char kKeyType[] = "key_type";
 const char kOriginId[] = "origin_id";
 
-bool GetMediaDrmKeyTypeFromDict(const base::Value& dict,
+bool GetMediaDrmKeyTypeFromDict(const base::Value::Dict& dict,
                                 media::MediaDrmKeyType* value_out) {
-  DCHECK(dict.is_dict());
   DCHECK(value_out);
 
-  const base::Value* value =
-      dict.FindKeyOfType(kKeyType, base::Value::Type::INTEGER);
+  const absl::optional<int> value = dict.FindInt(kKeyType);
   if (!value)
     return false;
 
-  int key_type = value->GetInt();
+  int key_type = *value;
   if (key_type < static_cast<int>(media::MediaDrmKeyType::UNKNOWN) ||
       key_type > static_cast<int>(media::MediaDrmKeyType::MAX)) {
     DVLOG(1) << "Corrupted key type.";
@@ -100,32 +100,29 @@ bool GetMediaDrmKeyTypeFromDict(const base::Value& dict,
   return true;
 }
 
-bool GetStringFromDict(const base::Value& dict,
+bool GetStringFromDict(const base::Value::Dict& dict,
                        const std::string& key,
                        std::string* value_out) {
-  DCHECK(dict.is_dict());
   DCHECK(value_out);
 
-  const base::Value* value = dict.FindKeyOfType(key, base::Value::Type::STRING);
+  const std::string* value = dict.FindString(key);
   if (!value)
     return false;
 
-  *value_out = value->GetString();
+  *value_out = *value;
   return true;
 }
 
 // Extract base::Time from |dict| with key kCreationTime. Returns true if |dict|
 // contains a valid time value.
-bool GetCreationTimeFromDict(const base::Value& dict, base::Time* time) {
-  DCHECK(dict.is_dict());
+bool GetCreationTimeFromDict(const base::Value::Dict& dict, base::Time* time) {
   DCHECK(time);
 
-  const base::Value* time_value =
-      dict.FindKeyOfType(kCreationTime, base::Value::Type::DOUBLE);
+  const absl::optional<double> time_value = dict.FindDouble(kCreationTime);
   if (!time_value)
     return false;
 
-  base::Time time_maybe_null = base::Time::FromDoubleT(time_value->GetDouble());
+  base::Time time_maybe_null = base::Time::FromDoubleT(*time_value);
   if (time_maybe_null.is_null())
     return false;
 
@@ -145,11 +142,11 @@ class OriginData {
 
   base::Time provision_time() const { return provision_time_; }
 
-  base::Value ToDictValue() const {
-    base::Value dict(base::Value::Type::DICTIONARY);
+  base::Value::Dict ToDictValue() const {
+    base::Value::Dict dict;
 
-    dict.SetKey(kOriginId, base::UnguessableTokenToValue(origin_id_));
-    dict.SetKey(kCreationTime, base::Value(provision_time_.ToDoubleT()));
+    dict.Set(kOriginId, base::UnguessableTokenToValue(origin_id_));
+    dict.Set(kCreationTime, base::Value(provision_time_.ToDoubleT()));
 
     return dict;
   }
@@ -158,12 +155,9 @@ class OriginData {
   // related to origin provision. Return nullptr if |origin_dict| has any
   // corruption, e.g. format error, missing fields, invalid value.
   static std::unique_ptr<OriginData> FromDictValue(
-      const base::Value& origin_dict) {
-    DCHECK(origin_dict.is_dict());
-
-    const base::Value* origin_id_value =
-        origin_dict.FindKeyOfType(kOriginId, base::Value::Type::STRING);
-    if (!origin_id_value)
+      const base::Value::Dict& origin_dict) {
+    const base::Value* origin_id_value = origin_dict.Find(kOriginId);
+    if (!origin_id_value || !origin_id_value->is_string())
       return nullptr;
 
     absl::optional<base::UnguessableToken> origin_id =
@@ -196,16 +190,15 @@ class SessionData {
 
   base::Time creation_time() const { return creation_time_; }
 
-  base::Value ToDictValue() const {
-    base::Value dict(base::Value::Type::DICTIONARY);
+  base::Value::Dict ToDictValue() const {
+    base::Value::Dict dict;
 
-    dict.SetKey(kKeySetId,
-                base::Value(std::string(
-                    reinterpret_cast<const char*>(key_set_id_.data()),
-                    key_set_id_.size())));
-    dict.SetKey(kMimeType, base::Value(mime_type_));
-    dict.SetKey(kKeyType, base::Value(static_cast<int>(key_type_)));
-    dict.SetKey(kCreationTime, base::Value(creation_time_.ToDoubleT()));
+    dict.Set(kKeySetId, base::Value(std::string(
+                            reinterpret_cast<const char*>(key_set_id_.data()),
+                            key_set_id_.size())));
+    dict.Set(kMimeType, base::Value(mime_type_));
+    dict.Set(kKeyType, base::Value(static_cast<int>(key_type_)));
+    dict.Set(kCreationTime, base::Value(creation_time_.ToDoubleT()));
 
     return dict;
   }
@@ -218,9 +211,7 @@ class SessionData {
   // for an offline license session. Return nullptr if |session_dict| has any
   // corruption, e.g. format error, missing fields, invalid data.
   static std::unique_ptr<SessionData> FromDictValue(
-      const base::Value& session_dict) {
-    DCHECK(session_dict.is_dict());
-
+      const base::Value::Dict& session_dict) {
     std::string key_set_id_string;
     if (!GetStringFromDict(session_dict, kKeySetId, &key_set_id_string))
       return nullptr;
@@ -263,25 +254,16 @@ class SessionData {
 
 // Get sessions dict for |origin_string| in |storage_dict|. This is a helper
 // function that works for both const and non const access.
-template <typename DictValue>
-DictValue* GetSessionsDictFromStorageDict(DictValue* storage_dict,
-                                          const std::string& origin_string) {
-  if (storage_dict == nullptr) {
-    DVLOG(1) << __func__ << ": No storage dict for origin " << origin_string;
-    return nullptr;
-  }
-
-  DCHECK(storage_dict->is_dict());
-
-  DictValue* origin_dict =
-      storage_dict->FindKeyOfType(origin_string, base::Value::Type::DICTIONARY);
+template <typename Dict>
+Dict* GetSessionsDictFromStorageDict(Dict& storage_dict,
+                                     const std::string& origin_string) {
+  Dict* origin_dict = storage_dict.FindDict(origin_string);
   if (!origin_dict) {
     DVLOG(1) << __func__ << ": No entry for origin " << origin_string;
     return nullptr;
   }
 
-  DictValue* sessions_dict =
-      origin_dict->FindKeyOfType(kSessions, base::Value::Type::DICTIONARY);
+  Dict* sessions_dict = origin_dict->FindDict(kSessions);
   if (!sessions_dict) {
     DVLOG(1) << __func__ << ": No sessions entry for origin " << origin_string;
     return nullptr;
@@ -293,29 +275,26 @@ DictValue* GetSessionsDictFromStorageDict(DictValue* storage_dict,
 // Create origin dict with |origin_id|, current time as creation time, and empty
 // sessions dict. It returns the sessions dict for caller to write session
 // information. Note that this clears any existing session information.
-base::Value* CreateOriginDictAndReturnSessionsDict(
-    base::Value* storage_dict,
+base::Value::Dict& CreateOriginDictAndReturnSessionsDict(
+    base::Value::Dict& storage_dict,
     const url::Origin& origin,
     const base::UnguessableToken& origin_id) {
-  DCHECK(storage_dict);
-  DCHECK(storage_dict->is_dict());
-
   return storage_dict
-      ->SetKey(origin.Serialize(), OriginData(origin_id).ToDictValue())
-      ->SetKey(kSessions, base::Value(base::Value::Type::DICTIONARY));
+      .Set(origin.Serialize(), OriginData(origin_id).ToDictValue())
+      ->GetDict()
+      .Set(kSessions, base::Value::Dict())
+      ->GetDict();
 }
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 // Clear sessions whose creation time falls in [start, end] from
 // |sessions_dict|. This function also cleans corruption data and should never
 // fail.
-void ClearSessionDataForTimePeriod(base::Value* sessions_dict,
+void ClearSessionDataForTimePeriod(base::Value::Dict& sessions_dict,
                                    base::Time start,
                                    base::Time end) {
-  DCHECK(sessions_dict->is_dict());
-
   std::vector<std::string> sessions_to_clear;
-  for (const auto key_value : sessions_dict->DictItems()) {
+  for (const auto key_value : sessions_dict) {
     const std::string& session_id = key_value.first;
 
     base::Value* session_dict = &key_value.second;
@@ -327,7 +306,7 @@ void ClearSessionDataForTimePeriod(base::Value* sessions_dict,
     }
 
     std::unique_ptr<SessionData> session_data =
-        SessionData::FromDictValue(*session_dict);
+        SessionData::FromDictValue(session_dict->GetDict());
     if (!session_data) {
       DLOG(WARNING) << "Session data for " << session_id
                     << " is corrupted, removing.";
@@ -344,7 +323,7 @@ void ClearSessionDataForTimePeriod(base::Value* sessions_dict,
 
   // Remove session data.
   for (const auto& session_id : sessions_to_clear)
-    sessions_dict->RemoveKey(session_id);
+    sessions_dict.Remove(session_id);
 }
 
 // 1. Removes the session data from origin dict if the session's creation time
@@ -352,14 +331,14 @@ void ClearSessionDataForTimePeriod(base::Value* sessions_dict,
 // 2. Removes the origin data if all of the sessions are removed.
 // 3. Returns a list of origin IDs to unprovision.
 std::vector<base::UnguessableToken> ClearMatchingLicenseData(
-    base::DictionaryValue* storage_dict,
+    base::Value::Dict& storage_dict,
     base::Time start,
     base::Time end,
     const base::RepeatingCallback<bool(const GURL&)>& filter) {
   std::vector<std::string> origins_to_delete;
   std::vector<base::UnguessableToken> origin_ids_to_unprovision;
 
-  for (const auto key_value : storage_dict->DictItems()) {
+  for (const auto key_value : storage_dict) {
     const std::string& origin_str = key_value.first;
 
     if (filter && !filter.Run(GURL(origin_str)))
@@ -374,7 +353,7 @@ std::vector<base::UnguessableToken> ClearMatchingLicenseData(
     }
 
     std::unique_ptr<OriginData> origin_data =
-        OriginData::FromDictValue(*origin_dict);
+        OriginData::FromDictValue(origin_dict->GetDict());
     if (!origin_data) {
       DLOG(WARNING) << "Origin data for " << origin_str
                     << " is corrupted, removing.";
@@ -385,8 +364,7 @@ std::vector<base::UnguessableToken> ClearMatchingLicenseData(
     if (origin_data->provision_time() > end)
       continue;
 
-    base::Value* sessions =
-        origin_dict->FindKeyOfType(kSessions, base::Value::Type::DICTIONARY);
+    base::Value::Dict* sessions = origin_dict->GetDict().FindDict(kSessions);
     if (!sessions) {
       // The origin is provisioned, but no persistent license is installed.
       origins_to_delete.push_back(origin_str);
@@ -394,9 +372,9 @@ std::vector<base::UnguessableToken> ClearMatchingLicenseData(
       continue;
     }
 
-    ClearSessionDataForTimePeriod(sessions, start, end);
+    ClearSessionDataForTimePeriod(*sessions, start, end);
 
-    if (sessions->DictEmpty()) {
+    if (sessions->empty()) {
       // Session data will be removed when removing origin data.
       origins_to_delete.push_back(origin_str);
       origin_ids_to_unprovision.push_back(origin_data->origin_id());
@@ -405,7 +383,7 @@ std::vector<base::UnguessableToken> ClearMatchingLicenseData(
 
   // Remove origin data.
   for (const auto& origin_str : origins_to_delete)
-    storage_dict->RemoveKey(origin_str);
+    storage_dict.Remove(origin_str);
 
   return origin_ids_to_unprovision;
 }
@@ -430,22 +408,21 @@ void ClearMediaDrmLicensesBlocking(
     media_drm_bridge->Unprovision();
   }
 }
-#endif  // defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_ANDROID)
 
 // Returns true if any session in |sessions_dict| has been modified more
 // recently than |start| and before |end|, and otherwise
 // returns false.
-bool SessionsModifiedBetween(const base::Value* sessions_dict,
+bool SessionsModifiedBetween(const base::Value::Dict& sessions_dict,
                              base::Time start,
                              base::Time end) {
-  DCHECK(sessions_dict->is_dict());
-  for (const auto key_value : sessions_dict->DictItems()) {
+  for (const auto key_value : sessions_dict) {
     const base::Value* session_dict = &key_value.second;
     if (!session_dict->is_dict())
       continue;
 
     std::unique_ptr<SessionData> session_data =
-        SessionData::FromDictValue(*session_dict);
+        SessionData::FromDictValue(session_dict->GetDict());
     if (!session_data)
       continue;
 
@@ -460,12 +437,10 @@ bool SessionsModifiedBetween(const base::Value* sessions_dict,
 // Returns the origin ID for |origin|, if it exists. Will return an empty value
 // if the origin ID can not be found in |storage_dict|.
 base::UnguessableToken GetOriginIdForOrigin(
-    const base::DictionaryValue* storage_dict,
+    const base::Value::Dict& storage_dict,
     const url::Origin& origin) {
-  DCHECK(storage_dict);
-
-  const base::Value* origin_dict = storage_dict->FindKeyOfType(
-      origin.Serialize(), base::Value::Type::DICTIONARY);
+  const base::Value::Dict* origin_dict =
+      storage_dict.FindDict(origin.Serialize());
   if (!origin_dict)
     return base::UnguessableToken::Null();
 
@@ -496,7 +471,7 @@ class InitializationSerializer {
              std::tie(other.pref_service, other.origin);
     }
 
-    PrefService* pref_service;
+    raw_ptr<PrefService> pref_service;
     const url::Origin origin;
   };
 
@@ -509,6 +484,10 @@ class InitializationSerializer {
   }
 
   InitializationSerializer() = default;
+
+  InitializationSerializer(const InitializationSerializer&) = delete;
+  InitializationSerializer& operator=(const InitializationSerializer&) = delete;
+
   ~InitializationSerializer() = default;
 
   void FetchOriginId(
@@ -520,8 +499,8 @@ class InitializationSerializer {
     DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 
     // Check if the preference has an existing origin ID.
-    const base::DictionaryValue* storage_dict =
-        pref_service->GetDictionary(prefs::kMediaDrmStorage);
+    const base::Value::Dict& storage_dict =
+        pref_service->GetDict(prefs::kMediaDrmStorage);
     base::UnguessableToken origin_id =
         GetOriginIdForOrigin(storage_dict, origin);
     if (origin_id) {
@@ -568,7 +547,7 @@ class InitializationSerializer {
 
     // Save the origin ID in the preference as long as it is not null.
     if (origin_id) {
-      DictionaryPrefUpdate update(pref_service, prefs::kMediaDrmStorage);
+      ScopedDictPrefUpdate update(pref_service, prefs::kMediaDrmStorage);
       CreateOriginDictAndReturnSessionsDict(update.Get(), origin,
                                             origin_id.value());
     }
@@ -592,7 +571,6 @@ class InitializationSerializer {
       pending_requests_;
 
   THREAD_CHECKER(thread_checker_);
-  DISALLOW_COPY_AND_ASSIGN(InitializationSerializer);
 };
 
 }  // namespace
@@ -607,13 +585,11 @@ std::set<GURL> MediaDrmStorageImpl::GetAllOrigins(
     const PrefService* pref_service) {
   DCHECK(pref_service);
 
-  const base::DictionaryValue* storage_dict =
-      pref_service->GetDictionary(prefs::kMediaDrmStorage);
-  if (!storage_dict)
-    return std::set<GURL>();
+  const base::Value::Dict& storage_dict =
+      pref_service->GetDict(prefs::kMediaDrmStorage);
 
   std::set<GURL> origin_set;
-  for (const auto key_value : storage_dict->DictItems()) {
+  for (const auto key_value : storage_dict) {
     GURL origin(key_value.first);
     if (origin.is_valid())
       origin_set.insert(origin);
@@ -629,16 +605,14 @@ std::vector<GURL> MediaDrmStorageImpl::GetOriginsModifiedBetween(
     base::Time end) {
   DCHECK(pref_service);
 
-  const base::DictionaryValue* storage_dict =
-      pref_service->GetDictionary(prefs::kMediaDrmStorage);
-  if (!storage_dict)
-    return {};
+  const base::Value::Dict& storage_dict =
+      pref_service->GetDict(prefs::kMediaDrmStorage);
 
   // Check each origin to see if it has been modified after |start| and
   // before |end|. If there are any errors in prefs::kMediaDrmStorage,
   // ignore them.
   std::vector<GURL> matching_origins;
-  for (const auto key_value : storage_dict->DictItems()) {
+  for (const auto key_value : storage_dict) {
     GURL origin(key_value.first);
     if (!origin.is_valid())
       continue;
@@ -648,7 +622,7 @@ std::vector<GURL> MediaDrmStorageImpl::GetOriginsModifiedBetween(
       continue;
 
     std::unique_ptr<OriginData> origin_data =
-        OriginData::FromDictValue(*origin_dict);
+        OriginData::FromDictValue(origin_dict->GetDict());
     if (!origin_data)
       continue;
 
@@ -656,13 +630,13 @@ std::vector<GURL> MediaDrmStorageImpl::GetOriginsModifiedBetween(
     // after |start|.
     if (origin_data->provision_time() < start) {
       // See if any session created recently.
-      const base::Value* sessions =
-          origin_dict->FindKeyOfType(kSessions, base::Value::Type::DICTIONARY);
+      const base::Value::Dict* sessions =
+          origin_dict->GetDict().FindDict(kSessions);
       if (!sessions)
         continue;
 
       // If no sessions modified recently, move on to the next origin.
-      if (!SessionsModifiedBetween(sessions, start, end))
+      if (!SessionsModifiedBetween(*sessions, start, end))
         continue;
     }
 
@@ -678,7 +652,7 @@ std::vector<GURL> MediaDrmStorageImpl::GetOriginsModifiedBetween(
   return matching_origins;
 }
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 // static
 void MediaDrmStorageImpl::ClearMatchingLicenses(
     PrefService* pref_service,
@@ -688,7 +662,7 @@ void MediaDrmStorageImpl::ClearMatchingLicenses(
     base::OnceClosure complete_cb) {
   DVLOG(1) << __func__ << ": Clear licenses [" << start << ", " << end << "]";
 
-  DictionaryPrefUpdate update(pref_service, prefs::kMediaDrmStorage);
+  ScopedDictPrefUpdate update(pref_service, prefs::kMediaDrmStorage);
 
   std::vector<base::UnguessableToken> no_license_origin_ids =
       ClearMatchingLicenseData(update.Get(), start, end, filter);
@@ -712,12 +686,12 @@ void MediaDrmStorageImpl::ClearMatchingLicenses(
 // MediaDrmStorageImpl
 
 MediaDrmStorageImpl::MediaDrmStorageImpl(
-    content::RenderFrameHost* render_frame_host,
+    content::RenderFrameHost& render_frame_host,
     PrefService* pref_service,
     GetOriginIdCB get_origin_id_cb,
     AllowEmptyOriginIdCB allow_empty_origin_id_cb,
     mojo::PendingReceiver<media::mojom::MediaDrmStorage> receiver)
-    : DocumentServiceBase(render_frame_host, std::move(receiver)),
+    : DocumentService(render_frame_host, std::move(receiver)),
       pref_service_(pref_service),
       get_origin_id_cb_(get_origin_id_cb),
       allow_empty_origin_id_cb_(allow_empty_origin_id_cb) {
@@ -730,15 +704,13 @@ MediaDrmStorageImpl::MediaDrmStorageImpl(
 }
 
 MediaDrmStorageImpl::MediaDrmStorageImpl(
-    content::RenderFrameHost* render_frame_host,
+    content::RenderFrameHost& render_frame_host,
     GetOriginIdCB get_origin_id_cb,
     AllowEmptyOriginIdCB allow_empty_origin_id_cb,
     mojo::PendingReceiver<media::mojom::MediaDrmStorage> receiver)
     : MediaDrmStorageImpl(
           render_frame_host,
-          user_prefs::UserPrefs::Get(
-              content::WebContents::FromRenderFrameHost(render_frame_host)
-                  ->GetBrowserContext()),
+          user_prefs::UserPrefs::Get(render_frame_host.GetBrowserContext()),
           get_origin_id_cb,
           allow_empty_origin_id_cb,
           std::move(receiver)) {}
@@ -812,14 +784,12 @@ void MediaDrmStorageImpl::OnProvisioned(OnProvisionedCallback callback) {
     return;
   }
 
-  DictionaryPrefUpdate update(pref_service_, prefs::kMediaDrmStorage);
-  base::DictionaryValue* storage_dict = update.Get();
-  DCHECK(storage_dict);
+  ScopedDictPrefUpdate update(pref_service_, prefs::kMediaDrmStorage);
 
   // Update origin dict once origin provisioning completes. There may be
   // orphaned session info from a previous provisioning. Clear them by
   // recreating the dicts.
-  CreateOriginDictAndReturnSessionsDict(storage_dict, origin(),
+  CreateOriginDictAndReturnSessionsDict(update.Get(), origin(),
                                         origin_id_.value());
   std::move(callback).Run(true);
 }
@@ -844,12 +814,11 @@ void MediaDrmStorageImpl::SavePersistentSession(
     return;
   }
 
-  DictionaryPrefUpdate update(pref_service_, prefs::kMediaDrmStorage);
-  base::DictionaryValue* storage_dict = update.Get();
-  DCHECK(storage_dict);
+  ScopedDictPrefUpdate update(pref_service_, prefs::kMediaDrmStorage);
+  base::Value::Dict& storage_dict = update.Get();
 
-  base::Value* sessions_dict = GetSessionsDictFromStorageDict<base::Value>(
-      storage_dict, origin().Serialize());
+  base::Value::Dict* sessions_dict =
+      GetSessionsDictFromStorageDict(storage_dict, origin().Serialize());
 
   // This could happen if the profile is removed, but the device is still
   // provisioned for the origin. In this case, just create a new entry.
@@ -857,12 +826,12 @@ void MediaDrmStorageImpl::SavePersistentSession(
   // branch. Deleting the profile causes reprovisioning of the origin.
   if (!sessions_dict) {
     DVLOG(1) << __func__ << ": No entry for origin " << origin();
-    sessions_dict = CreateOriginDictAndReturnSessionsDict(
+    sessions_dict = &CreateOriginDictAndReturnSessionsDict(
         storage_dict, origin(), origin_id_.value());
     DCHECK(sessions_dict);
   }
 
-  DVLOG_IF(1, sessions_dict->FindKey(session_id))
+  DVLOG_IF(1, sessions_dict->contains(session_id))
       << __func__ << ": Session ID already exists and will be replaced.";
 
   // The key type of the session should be valid. MediaDrmKeyType::MIN/UNKNOWN
@@ -870,7 +839,7 @@ void MediaDrmStorageImpl::SavePersistentSession(
   DCHECK_GT(session_data->key_type, media::MediaDrmKeyType::MIN);
   DCHECK_LE(session_data->key_type, media::MediaDrmKeyType::MAX);
 
-  sessions_dict->SetKey(
+  sessions_dict->Set(
       session_id, SessionData(session_data->key_set_id, session_data->mime_type,
                               session_data->key_type)
                       .ToDictValue());
@@ -897,17 +866,14 @@ void MediaDrmStorageImpl::LoadPersistentSession(
     return;
   }
 
-  const base::Value* sessions_dict =
-      GetSessionsDictFromStorageDict<const base::Value>(
-          pref_service_->GetDictionary(prefs::kMediaDrmStorage),
-          origin().Serialize());
+  const base::Value::Dict* sessions_dict = GetSessionsDictFromStorageDict(
+      pref_service_->GetDict(prefs::kMediaDrmStorage), origin().Serialize());
   if (!sessions_dict) {
     std::move(callback).Run(nullptr);
     return;
   }
 
-  const base::Value* session_dict =
-      sessions_dict->FindKeyOfType(session_id, base::Value::Type::DICTIONARY);
+  const base::Value::Dict* session_dict = sessions_dict->FindDict(session_id);
   if (!session_dict) {
     DVLOG(1) << __func__ << ": No session " << session_id << " for origin "
              << origin();
@@ -945,17 +911,17 @@ void MediaDrmStorageImpl::RemovePersistentSession(
     return;
   }
 
-  DictionaryPrefUpdate update(pref_service_, prefs::kMediaDrmStorage);
+  ScopedDictPrefUpdate update(pref_service_, prefs::kMediaDrmStorage);
 
-  base::Value* sessions_dict = GetSessionsDictFromStorageDict<base::Value>(
-      update.Get(), origin().Serialize());
+  base::Value::Dict* sessions_dict =
+      GetSessionsDictFromStorageDict(update.Get(), origin().Serialize());
 
   if (!sessions_dict) {
     std::move(callback).Run(true);
     return;
   }
 
-  sessions_dict->RemoveKey(session_id);
+  sessions_dict->Remove(session_id);
   std::move(callback).Run(true);
 }
 
