@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,18 +11,22 @@ namespace content_capture {
 
 namespace features {
 
-extern const base::Feature kContentCapture;
+BASE_DECLARE_FEATURE(kContentCapture);
 
 // ContentCapture is triggered in the unpredictable conditions which might be
 // changed on different aiai release or configuration push, this feature allows
 // us to trigger the ContentCapture independently to get the unbiased result.
-extern const base::Feature kContentCaptureTriggeringForExperiment;
+BASE_DECLARE_FEATURE(kContentCaptureTriggeringForExperiment);
+
+// ContentCapture in WebLayer, this flag is independent from the kContentCapture
+// flag.
+BASE_DECLARE_FEATURE(kContentCaptureInWebLayer);
 
 bool IsContentCaptureEnabled();
 bool ShouldTriggerContentCaptureForExperiment();
+bool IsContentCaptureEnabledInWebLayer();
 
-int TaskLongDelayInMilliseconds();
-int TaskShortDelayInMilliseconds();
+int TaskInitialDelayInMilliseconds();
 
 }  // namespace features
 
