@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,80 +6,59 @@
 
 #include <utility>
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/command_line.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/notreached.h"
+#include "base/observer_list.h"
 #include "base/path_service.h"
-#include "base/task/post_task.h"
+#include "base/strings/strcat.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/task/thread_pool.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "components/enterprise/browser/controller/browser_dm_token_storage.h"
 #include "components/enterprise/browser/controller/chrome_browser_cloud_management_helper.h"
 #include "components/enterprise/browser/enterprise_switches.h"
-#include "components/enterprise/browser/reporting/report_generator.h"
-#include "components/enterprise/browser/reporting/report_scheduler.h"
 #include "components/policy/core/browser/browser_policy_connector.h"
-#include "components/policy/core/common/cloud/chrome_browser_cloud_management_metrics.h"
+#include "components/policy/core/common/cloud/client_data_delegate.h"
 #include "components/policy/core/common/cloud/cloud_external_data_manager.h"
 #include "components/policy/core/common/cloud/cloud_policy_constants.h"
 #include "components/policy/core/common/cloud/dm_token.h"
 #include "components/policy/core/common/cloud/machine_level_user_cloud_policy_manager.h"
 #include "components/policy/core/common/cloud/machine_level_user_cloud_policy_store.h"
 #include "components/policy/core/common/configuration_policy_provider.h"
-#include "components/policy/core/common/features.h"
-#include "components/policy/core/common/policy_types.h"
-#include "components/policy/policy_constants.h"
+#include "components/policy/core/common/policy_logger.h"
+#include "components/prefs/pref_service.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 
+#if !BUILDFLAG(IS_ANDROID)
+#include "components/enterprise/browser/reporting/report_generator.h"
+#include "components/enterprise/browser/reporting/report_scheduler.h"
+#endif  // !BUILDFLAG(IS_ANDROID)
+
 namespace policy {
 
-namespace {
-
-void RecordEnrollmentResult(
-    ChromeBrowserCloudManagementEnrollmentResult result) {
-  UMA_HISTOGRAM_ENUMERATION(
-      "Enterprise.MachineLevelUserCloudPolicyEnrollment.Result", result);
-}
-
-// Read the kCloudPolicyOverridesPlatformPolicy from platform provider directly
-// because the local_state is not ready when the
-// MachineLevelUserCloudPolicyManager is created.
-bool DoesCloudPolicyHasPriority(
-    ConfigurationPolicyProvider* platform_provider) {
-  if (!platform_provider)
-    return false;
-  const auto* entry =
-      platform_provider->policies()
-          .Get(PolicyNamespace(POLICY_DOMAIN_CHROME, std::string()))
-          .Get(key::kCloudPolicyOverridesPlatformPolicy);
-  if (!entry || entry->scope == POLICY_SCOPE_USER ||
-      entry->level == POLICY_LEVEL_RECOMMENDED)
-    return false;
-
-  return entry->value()->is_bool() && entry->value()->GetBool();
-}
-
-}  // namespace
-
 const base::FilePath::CharType
     ChromeBrowserCloudManagementController::kPolicyDir[] =
         FILE_PATH_LITERAL("Policy");
 
-bool ChromeBrowserCloudManagementController::IsEnabled() {
-#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
-  return true;
-#else
-  return base::CommandLine::ForCurrentProcess()->HasSwitch(
-      switches::kEnableChromeBrowserCloudManagement);
-#endif
+std::unique_ptr<enterprise_connectors::DeviceTrustKeyManager>
+ChromeBrowserCloudManagementController::Delegate::
+    CreateDeviceTrustKeyManager() {
+  return nullptr;
+}
+
+void ChromeBrowserCloudManagementController::Delegate::DeferInitialization(
+    base::OnceClosure callback) {
+  NOTREACHED();
 }
 
 ChromeBrowserCloudManagementController::ChromeBrowserCloudManagementController(
     std::unique_ptr<ChromeBrowserCloudManagementController::Delegate> delegate)
-    : delegate_(std::move(delegate)) {
+    : delegate_(std::move(delegate)),
+      client_data_delegate_(delegate_->CreateClientDataDelegate()) {
   delegate_->SetDMTokenStorageDelegate();
 }
 
@@ -91,11 +70,24 @@ ChromeBrowserCloudManagementController::
     cloud_policy_client_->RemoveObserver(this);
 }
 
+// static
+bool ChromeBrowserCloudManagementController::IsEnabled() {
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
+  return true;
+#else
+  return base::CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kEnableChromeBrowserCloudManagement);
+#endif
+}
+
 std::unique_ptr<MachineLevelUserCloudPolicyManager>
 ChromeBrowserCloudManagementController::CreatePolicyManager(
     ConfigurationPolicyProvider* platform_provider) {
-  if (!IsEnabled())
+  if (!IsEnabled()) {
+    LOG_POLICY(WARNING, CBCM_ENROLLMENT)
+        << "Could not create policy manager as CBCM is not enabled.";
     return nullptr;
+  }
 
   std::string enrollment_token =
       BrowserDMTokenStorage::Get()->RetrieveEnrollmentToken();
@@ -103,14 +95,14 @@ ChromeBrowserCloudManagementController::CreatePolicyManager(
   std::string client_id = BrowserDMTokenStorage::Get()->RetrieveClientId();
 
   if (dm_token.is_empty())
-    VLOG(1) << "DM token = none";
+    VLOG_POLICY(1, CBCM_ENROLLMENT) << "DM token = empty";
   else if (dm_token.is_invalid())
-    VLOG(1) << "DM token = invalid";
+    VLOG_POLICY(1, CBCM_ENROLLMENT) << "DM token = invalid";
   else if (dm_token.is_valid())
-    VLOG(1) << "DM token = from persistence";
+    VLOG_POLICY(1, CBCM_ENROLLMENT) << "DM token = from persistence";
 
-  VLOG(1) << "Enrollment token = " << enrollment_token;
-  VLOG(1) << "Client ID = " << client_id;
+  VLOG_POLICY(1, CBCM_ENROLLMENT) << "Enrollment token = " << enrollment_token;
+  VLOG_POLICY(1, CBCM_ENROLLMENT) << "Client ID = " << client_id;
 
   // Don't create the policy manager if the DM token is explicitly invalid or if
   // both tokens are empty.
@@ -123,24 +115,17 @@ ChromeBrowserCloudManagementController::CreatePolicyManager(
   if (!base::PathService::Get(delegate_->GetUserDataDirKey(), &user_data_dir))
     return nullptr;
 
-  DVLOG(1) << "Creating machine level user cloud policy manager";
-
-  bool cloud_policy_has_priority =
-      DoesCloudPolicyHasPriority(platform_provider);
-  if (cloud_policy_has_priority) {
-    DVLOG(1) << "Cloud policies are now overriding platform policies with "
-                "machine scope.";
-  }
+  DVLOG_POLICY(1, CBCM_ENROLLMENT)
+      << "Creating machine level user cloud policy manager";
 
   base::FilePath policy_dir =
       user_data_dir.Append(ChromeBrowserCloudManagementController::kPolicyDir);
 
-  base::FilePath external_policy_path = delegate_->GetExternalPolicyPath();
+  base::FilePath external_policy_path = delegate_->GetExternalPolicyDir();
 
   std::unique_ptr<MachineLevelUserCloudPolicyStore> policy_store =
       MachineLevelUserCloudPolicyStore::Create(
           dm_token, client_id, external_policy_path, policy_dir,
-          cloud_policy_has_priority,
           base::ThreadPool::CreateSequencedTaskRunner(
               {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
                // Block shutdown to make sure the policy cache update is always
@@ -148,19 +133,49 @@ ChromeBrowserCloudManagementController::CreatePolicyManager(
                base::TaskShutdownBehavior::BLOCK_SHUTDOWN}));
   return std::make_unique<MachineLevelUserCloudPolicyManager>(
       std::move(policy_store), nullptr, policy_dir,
-      base::ThreadTaskRunnerHandle::Get(),
+      base::SingleThreadTaskRunner::GetCurrentDefault(),
       delegate_->CreateNetworkConnectionTrackerGetter());
 }
 
+void ChromeBrowserCloudManagementController::DeferrableCreatePolicyManager(
+    ConfigurationPolicyProvider* platform_provider,
+    base::OnceCallback<
+        void(std::unique_ptr<MachineLevelUserCloudPolicyManager>)> callback) {
+  if (delegate_->ReadyToCreatePolicyManager()) {
+    DeferrableCreatePolicyManagerImpl(platform_provider, std::move(callback));
+  } else {
+    // Postpone policy manager creation to happen during controller
+    // initialization, so it's guaranteed that all dependencies have been
+    // resolved (e.g. on Android it depends on PolicyService being created
+    // and initialized).
+    create_cloud_policy_manager_callback_ = base::BindOnce(
+        &ChromeBrowserCloudManagementController::
+            DeferrableCreatePolicyManagerImpl,
+        weak_factory_.GetWeakPtr(), platform_provider, std::move(callback));
+  }
+}
+
 void ChromeBrowserCloudManagementController::Init(
     PrefService* local_state,
     scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory) {
-  if (!IsEnabled())
+  if (!IsEnabled()) {
+    LOG_POLICY(ERROR, CBCM_ENROLLMENT)
+        << "Cloud management controller initialization aborted as CBCM is not "
+           "enabled.";
     return;
+  }
+  LOG_POLICY(INFO, CBCM_ENROLLMENT)
+      << "Starting CBCM Controller Initialization";
 
-  if (base::FeatureList::IsEnabled(
-          policy::features::kCBCMPolicyInvalidations)) {
-    delegate_->InitializeOAuthTokenFactory(url_loader_factory, local_state);
+  delegate_->InitializeOAuthTokenFactory(url_loader_factory, local_state);
+
+  if (create_cloud_policy_manager_callback_) {
+    // The conditions that allow controller initialization should also unblock
+    // policy manager creation (e.g. on Android both depend on PolicyService
+    // being initialized).
+    DCHECK(delegate_->ReadyToCreatePolicyManager());
+
+    std::move(create_cloud_policy_manager_callback_).Run();
   }
 
   // Post the task of CreateReportScheduler to run on best effort after launch
@@ -176,8 +191,11 @@ void ChromeBrowserCloudManagementController::Init(
   DeviceManagementService* device_management_service =
       delegate_->GetDeviceManagementService();
 
-  if (!policy_manager)
+  if (!policy_manager) {
+    LOG_POLICY(INFO, CBCM_ENROLLMENT)
+        << "No machine level policy manager exists.";
     return;
+  }
 
   // If there exists an enrollment token, then there are three states:
   //   1/ There also exists a valid DM token.  This machine is already
@@ -190,10 +208,14 @@ void ChromeBrowserCloudManagementController::Init(
   std::string client_id;
   DMToken dm_token = BrowserDMTokenStorage::Get()->RetrieveDMToken();
 
-  if (dm_token.is_invalid())
+  if (dm_token.is_invalid()) {
+    LOG_POLICY(WARNING, CBCM_ENROLLMENT)
+        << "Enrollment Token exists but DM Token is Invalid";
     return;
+  }
 
   if (dm_token.is_valid()) {
+    VLOG_POLICY(1, CBCM_ENROLLMENT) << "Valid DM Token retrieved.";
     policy_fetcher_ = std::make_unique<MachineLevelUserCloudPolicyFetcher>(
         policy_manager, local_state, device_management_service,
         url_loader_factory);
@@ -221,9 +243,11 @@ void ChromeBrowserCloudManagementController::Init(
     enrollment_start_time_ = base::Time::Now();
 
     // Not registered already, so do it now.
+    LOG_POLICY(INFO, CBCM_ENROLLMENT)
+        << "Starting cloud management registration with an enrollment token.";
     cloud_management_registrar_->RegisterForCloudManagementWithEnrollmentToken(
-        enrollment_token, client_id,
-        base::BindRepeating(
+        enrollment_token, client_id, *client_data_delegate_,
+        base::BindOnce(
             &ChromeBrowserCloudManagementController::
                 RegisterForCloudManagementWithEnrollmentTokenCallback,
             weak_factory_.GetWeakPtr()));
@@ -239,6 +263,20 @@ void ChromeBrowserCloudManagementController::Init(
   }
 }
 
+void ChromeBrowserCloudManagementController::MaybeInit(
+    PrefService* local_state,
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory) {
+  if (delegate_->ReadyToInit()) {
+    Init(local_state, url_loader_factory);
+  } else {
+    LOG_POLICY(INFO, CBCM_ENROLLMENT)
+        << "Deferring CBCM Controller Initialization until it is unblocked.";
+    delegate_->DeferInitialization(base::BindOnce(
+        &ChromeBrowserCloudManagementController::Init,
+        weak_factory_.GetWeakPtr(), local_state, url_loader_factory));
+  }
+}
+
 bool ChromeBrowserCloudManagementController::
     WaitUntilPolicyEnrollmentFinished() {
   return delegate_->WaitUntilPolicyEnrollmentFinished();
@@ -258,11 +296,23 @@ bool ChromeBrowserCloudManagementController::
   return delegate_->IsEnterpriseStartupDialogShowing();
 }
 
-void ChromeBrowserCloudManagementController::UnenrollBrowser() {
+void ChromeBrowserCloudManagementController::UnenrollBrowser(
+    bool delete_dm_token) {
+  if (delete_dm_token) {
+    DVLOG_POLICY(1, CBCM_ENROLLMENT)
+        << "Browser unenrollment: Attempting DMToken deletion";
+    BrowserDMTokenStorage::Get()->ClearDMToken(base::BindOnce(
+        &ChromeBrowserCloudManagementController::UnenrollCallback,
+        weak_factory_.GetWeakPtr(), "DMTokenDeletion"));
+    return;
+  }
+
   // Invalidate DM token in storage.
-  BrowserDMTokenStorage::Get()->InvalidateDMToken(base::BindOnce(
-      &ChromeBrowserCloudManagementController::InvalidateDMTokenCallback,
-      weak_factory_.GetWeakPtr()));
+  DVLOG_POLICY(1, CBCM_ENROLLMENT)
+      << "Browser unenrollment: Attempting DMToken invalidation";
+  BrowserDMTokenStorage::Get()->InvalidateDMToken(
+      base::BindOnce(&ChromeBrowserCloudManagementController::UnenrollCallback,
+                     weak_factory_.GetWeakPtr(), "UnenrollSuccess"));
 }
 
 void ChromeBrowserCloudManagementController::InvalidatePolicies() {
@@ -278,17 +328,19 @@ void ChromeBrowserCloudManagementController::InvalidatePolicies() {
     report_scheduler_->OnDMTokenUpdated();
 }
 
-void ChromeBrowserCloudManagementController::InvalidateDMTokenCallback(
+void ChromeBrowserCloudManagementController::UnenrollCallback(
+    const std::string& metric_name,
     bool success) {
   UMA_HISTOGRAM_BOOLEAN(
-      "Enterprise.MachineLevelUserCloudPolicyEnrollment.UnenrollSuccess",
+      base::StrCat(
+          {"Enterprise.MachineLevelUserCloudPolicyEnrollment.", metric_name}),
       success);
-  if (success) {
-    DVLOG(1) << "Successfully invalidated the DM token";
+  DVLOG_POLICY(1, CBCM_ENROLLMENT)
+      << "Browser unenrollment: " << (success ? "succeeded" : "failed");
+
+  if (success)
     InvalidatePolicies();
-  } else {
-    DVLOG(1) << "Failed to invalidate the DM token";
-  }
+
   NotifyBrowserUnenrolled(success);
 }
 
@@ -305,9 +357,15 @@ void ChromeBrowserCloudManagementController::OnRegistrationStateChanged(
 void ChromeBrowserCloudManagementController::OnClientError(
     CloudPolicyClient* client) {
   // DM_STATUS_SERVICE_DEVICE_NOT_FOUND being the last status implies the
-  // browser has been unenrolled.
-  if (client->status() == DM_STATUS_SERVICE_DEVICE_NOT_FOUND)
-    UnenrollBrowser();
+  // browser has been unenrolled via DMToken invalidation, so it is not expected
+  // to re-enroll automatically. DM_STATUS_SERVICE_DEVICE_NEEDS_RESET signals
+  // that the browser has been unenrolled via DMToken deletion, and that it will
+  // automatically re-enroll if a valid enrollment token has been set.
+  if (client->last_dm_status() == DM_STATUS_SERVICE_DEVICE_NOT_FOUND ||
+      client->last_dm_status() == DM_STATUS_SERVICE_DEVICE_NEEDS_RESET) {
+    UnenrollBrowser(/*delete_dm_token=*/client->last_dm_status() ==
+                    DM_STATUS_SERVICE_DEVICE_NEEDS_RESET);
+  }
 }
 
 void ChromeBrowserCloudManagementController::OnServiceAccountSet(
@@ -317,11 +375,25 @@ void ChromeBrowserCloudManagementController::OnServiceAccountSet(
 }
 
 void ChromeBrowserCloudManagementController::ShutDown() {
+  NotifyShutdown();
   delegate_->ShutDown();
   if (report_scheduler_)
     report_scheduler_.reset();
 }
 
+enterprise_connectors::DeviceTrustKeyManager*
+ChromeBrowserCloudManagementController::GetDeviceTrustKeyManager() {
+  if (!device_trust_key_manager_) {
+    device_trust_key_manager_ = delegate_->CreateDeviceTrustKeyManager();
+  }
+  return device_trust_key_manager_.get();
+}
+
+void ChromeBrowserCloudManagementController::SetGaiaURLLoaderFactory(
+    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory) {
+  delegate_->SetGaiaURLLoaderFactory(url_loader_factory);
+}
+
 void ChromeBrowserCloudManagementController::NotifyPolicyRegisterFinished(
     bool succeeded) {
   for (auto& observer : observers_) {
@@ -337,7 +409,13 @@ void ChromeBrowserCloudManagementController::NotifyBrowserUnenrolled(
 
 void ChromeBrowserCloudManagementController::NotifyCloudReportingLaunched() {
   for (auto& observer : observers_) {
-    observer.OnCloudReportingLaunched();
+    observer.OnCloudReportingLaunched(report_scheduler_.get());
+  }
+}
+
+void ChromeBrowserCloudManagementController::NotifyShutdown() {
+  for (auto& observer : observers_) {
+    observer.OnShutdown();
   }
 }
 
@@ -359,7 +437,8 @@ void ChromeBrowserCloudManagementController::
   base::TimeDelta enrollment_time = base::Time::Now() - enrollment_start_time_;
 
   if (dm_token.empty()) {
-    VLOG(1) << "No DM token returned from browser registration.";
+    VLOG_POLICY(1, CBCM_ENROLLMENT)
+        << "No DM token returned from browser registration.";
     RecordEnrollmentResult(
         ChromeBrowserCloudManagementEnrollmentResult::kFailedToFetch);
     UMA_HISTOGRAM_TIMES(
@@ -374,7 +453,7 @@ void ChromeBrowserCloudManagementController::
     return;
   }
 
-  VLOG(1) << "DM token retrieved from server.";
+  VLOG_POLICY(1, CBCM_ENROLLMENT) << "DM token retrieved from server.";
 
   UMA_HISTOGRAM_TIMES(
       "Enterprise.MachineLevelUserCloudPolicyEnrollment.RequestSuccessTime",
@@ -382,20 +461,26 @@ void ChromeBrowserCloudManagementController::
 
   // TODO(alito): Log failures to store the DM token. Should we try again later?
   BrowserDMTokenStorage::Get()->StoreDMToken(
-      dm_token, base::BindOnce([](bool success) {
-        if (!success) {
-          DVLOG(1) << "Failed to store the DM token";
-          RecordEnrollmentResult(
-              ChromeBrowserCloudManagementEnrollmentResult::kFailedToStore);
-        } else {
-          DVLOG(1) << "Successfully stored the DM token";
-          RecordEnrollmentResult(
-              ChromeBrowserCloudManagementEnrollmentResult::kSuccess);
-        }
-      }));
+      dm_token,
+      base::BindOnce(
+          [](base::WeakPtr<ChromeBrowserCloudManagementController> controller,
+             bool success) {
+            if (!success) {
+              DVLOG_POLICY(1, CBCM_ENROLLMENT)
+                  << "Failed to store the DM token";
+              controller->RecordEnrollmentResult(
+                  ChromeBrowserCloudManagementEnrollmentResult::kFailedToStore);
+            } else {
+              DVLOG_POLICY(1, CBCM_ENROLLMENT)
+                  << "Successfully stored the DM token";
+              controller->RecordEnrollmentResult(
+                  ChromeBrowserCloudManagementEnrollmentResult::kSuccess);
+            }
+          },
+          weak_factory_.GetWeakPtr()));
 
   // Start fetching policies.
-  VLOG(1) << "Fetch policy after enrollment.";
+  VLOG_POLICY(1, POLICY_FETCHING) << "Fetch policy after enrollment.";
   policy_fetcher_->SetupRegistrationAndFetchPolicy(
       BrowserDMTokenStorage::Get()->RetrieveDMToken(), client_id);
   if (report_scheduler_) {
@@ -411,15 +496,39 @@ void ChromeBrowserCloudManagementController::CreateReportScheduler() {
       delegate_->GetSharedURLLoaderFactory(),
       CloudPolicyClient::DeviceDMTokenCallback());
   cloud_policy_client_->AddObserver(this);
-  report_scheduler_ =
-      delegate_->CreateReportScheduler(cloud_policy_client_.get());
+  auto reporting_delegate_factory = delegate_->GetReportingDelegateFactory();
+
+  enterprise_reporting::ReportScheduler::CreateParams params;
+  params.client = cloud_policy_client_.get();
+  params.delegate = reporting_delegate_factory->GetReportSchedulerDelegate();
+  params.report_generator =
+      std::make_unique<enterprise_reporting::ReportGenerator>(
+          reporting_delegate_factory.get());
+  params.real_time_report_generator =
+      std::make_unique<enterprise_reporting::RealTimeReportGenerator>(
+          reporting_delegate_factory.get());
+  report_scheduler_ = std::make_unique<enterprise_reporting::ReportScheduler>(
+      std::move(params));
 
   NotifyCloudReportingLaunched();
 }
 
-void ChromeBrowserCloudManagementController::SetGaiaURLLoaderFactory(
-    scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory) {
-  delegate_->SetGaiaURLLoaderFactory(url_loader_factory);
+void ChromeBrowserCloudManagementController::DeferrableCreatePolicyManagerImpl(
+    ConfigurationPolicyProvider* platform_provider,
+    base::OnceCallback<
+        void(std::unique_ptr<MachineLevelUserCloudPolicyManager>)> callback) {
+  std::unique_ptr<MachineLevelUserCloudPolicyManager> policy_manager =
+      CreatePolicyManager(platform_provider);
+  std::move(callback).Run(std::move(policy_manager));
+}
+
+void ChromeBrowserCloudManagementController::RecordEnrollmentResult(
+    ChromeBrowserCloudManagementEnrollmentResult result) {
+  UMA_HISTOGRAM_ENUMERATION(
+      "Enterprise.MachineLevelUserCloudPolicyEnrollment.Result", result);
+  for (auto& observer : observers_) {
+    observer.OnEnrollmentResultRecorded();
+  }
 }
 
 }  // namespace policy
