@@ -1,4 +1,4 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,9 +13,9 @@
 
 #include "base/containers/flat_set.h"
 #include "base/files/file_util.h"
-#include "base/macros.h"
+#include "base/functional/callback_forward.h"
+#include "base/pickle.h"
 #include "base/run_loop.h"
-#include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
 #include "cc/test/pixel_comparator.h"
 #include "cc/test/pixel_test_utils.h"
@@ -24,7 +24,9 @@
 #include "components/exo/data_offer_delegate.h"
 #include "components/exo/test/exo_test_base.h"
 #include "components/exo/test/exo_test_data_exchange_delegate.h"
+#include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "ui/base/clipboard/clipboard.h"
 #include "ui/base/clipboard/clipboard_format_type.h"
 #include "ui/base/clipboard/scoped_clipboard_writer.h"
 #include "ui/base/data_transfer_policy/data_transfer_policy_controller.h"
@@ -35,12 +37,21 @@
 namespace exo {
 namespace {
 
-using DataOfferTest = test::ExoTestBase;
+class DataOfferTest : public test::ExoTestBase {
+ public:
+  void TearDown() override {
+    ui::Clipboard::DestroyClipboardForCurrentThread();
+    test::ExoTestBase::TearDown();
+  }
+};
 
 class TestDataOfferDelegate : public DataOfferDelegate {
  public:
   TestDataOfferDelegate() {}
 
+  TestDataOfferDelegate(const TestDataOfferDelegate&) = delete;
+  TestDataOfferDelegate& operator=(const TestDataOfferDelegate&) = delete;
+
   // Called at the top of the data device's destructor, to give observers a
   // chance to remove themselves.
   void OnDataOfferDestroying(DataOffer* offer) override {}
@@ -69,8 +80,6 @@ class TestDataOfferDelegate : public DataOfferDelegate {
   base::flat_set<std::string> mime_types_;
   base::flat_set<DndAction> source_actions_;
   DndAction dnd_action_ = DndAction::kNone;
-
-  DISALLOW_COPY_AND_ASSIGN(TestDataOfferDelegate);
 };
 
 class TestDataTransferPolicyController : ui::DataTransferPolicyController {
@@ -85,19 +94,25 @@ class TestDataTransferPolicyController : ui::DataTransferPolicyController {
 
  private:
   // ui::DataTransferPolicyController:
-  bool IsClipboardReadAllowed(
-      const ui::DataTransferEndpoint* const data_src,
-      const ui::DataTransferEndpoint* const data_dst) override {
+  bool IsClipboardReadAllowed(const ui::DataTransferEndpoint* const data_src,
+                              const ui::DataTransferEndpoint* const data_dst,
+                              const absl::optional<size_t> size) override {
     if (data_src)
       last_src_type_ = data_src->type();
     last_dst_type_ = data_dst->type();
     return true;
   }
 
-  bool IsDragDropAllowed(const ui::DataTransferEndpoint* const data_src,
-                         const ui::DataTransferEndpoint* const data_dst,
-                         const bool is_drop) override {
-    return true;
+  void PasteIfAllowed(const ui::DataTransferEndpoint* const data_src,
+                      const ui::DataTransferEndpoint* const data_dst,
+                      const absl::optional<size_t> size,
+                      content::RenderFrameHost* web_contents,
+                      base::OnceCallback<void(bool)> callback) override {}
+
+  void DropIfAllowed(const ui::OSExchangeData* const drag_data,
+                     const ui::DataTransferEndpoint* const data_dst,
+                     base::OnceClosure drop_cb) override {
+    std::move(drop_cb).Run();
   }
 
   ui::EndpointType last_src_type_ = ui::EndpointType::kUnknownVm;
@@ -119,7 +134,7 @@ bool ReadString(base::ScopedFD fd, std::string* out) {
   return true;
 }
 
-bool ReadString16(base::ScopedFD fd, base::string16* out) {
+bool ReadString16(base::ScopedFD fd, std::u16string* out) {
   std::array<char, 128> buffer;
   char* it = buffer.begin();
   while (it != buffer.end()) {
@@ -129,8 +144,8 @@ bool ReadString16(base::ScopedFD fd, base::string16* out) {
       break;
     it += result;
   }
-  *out = base::string16(reinterpret_cast<base::char16*>(buffer.data()),
-                        (it - buffer.begin()) / sizeof(base::char16));
+  *out = std::u16string(reinterpret_cast<char16_t*>(buffer.data()),
+                        (it - buffer.begin()) / sizeof(char16_t));
   return true;
 }
 
@@ -140,7 +155,7 @@ TEST_F(DataOfferTest, SetTextDropData) {
   source_actions.insert(DndAction::kMove);
 
   ui::OSExchangeData data;
-  data.SetString(base::string16(base::ASCIIToUTF16("Test data")));
+  data.SetString(std::u16string(u"Test data"));
 
   TestDataOfferDelegate delegate;
   DataOffer data_offer(&delegate);
@@ -198,7 +213,7 @@ TEST_F(DataOfferTest, SetHTMLDropData) {
   ReadString(std::move(read), &result);
   EXPECT_EQ(result, html_data);
 
-  base::string16 result16;
+  std::u16string result16;
   EXPECT_TRUE(base::CreatePipe(&read, &write));
   data_offer.Receive("text/html;charset=utf-16", std::move(write));
   ReadString16(std::move(read), &result16);
@@ -238,13 +253,28 @@ TEST_F(DataOfferTest, SetPickleDropData) {
   EXPECT_EQ(1u, delegate.mime_types().count("text/uri-list"));
 }
 
+TEST_F(DataOfferTest, SetFileContentsDropData) {
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  ui::OSExchangeData data;
+  data.provider().SetFileContents(base::FilePath("\"test file\".jpg"),
+                                  std::string("test data"));
+  data_offer.SetDropData(&data_exchange_delegate, nullptr, data);
+
+  EXPECT_EQ(1u, delegate.mime_types().size());
+  EXPECT_EQ(1u, delegate.mime_types().count(
+                    "application/octet-stream;name=\"\\\"test file\\\".jpg\""));
+}
+
 TEST_F(DataOfferTest, ReceiveString) {
   TestDataOfferDelegate delegate;
   DataOffer data_offer(&delegate);
 
   TestDataExchangeDelegate data_exchange_delegate;
   ui::OSExchangeData data;
-  data.SetString(base::ASCIIToUTF16("Test data"));
+  data.SetString(u"Test data");
   data_offer.SetDropData(&data_exchange_delegate, nullptr, data);
 
   base::ScopedFD read_pipe;
@@ -260,9 +290,9 @@ TEST_F(DataOfferTest, ReceiveString) {
   base::ScopedFD write_pipe_16;
   ASSERT_TRUE(base::CreatePipe(&read_pipe_16, &write_pipe_16));
   data_offer.Receive("text/plain;charset=utf-16", std::move(write_pipe_16));
-  base::string16 result_16;
+  std::u16string result_16;
   ASSERT_TRUE(ReadString16(std::move(read_pipe_16), &result_16));
-  EXPECT_EQ(base::ASCIIToUTF16("Test data"), result_16);
+  EXPECT_EQ(u"Test data", result_16);
 
   base::ScopedFD read_pipe_8;
   base::ScopedFD write_pipe_8;
@@ -279,16 +309,16 @@ TEST_F(DataOfferTest, ReceiveHTML) {
 
   TestDataExchangeDelegate data_exchange_delegate;
   ui::OSExchangeData data;
-  data.SetHtml(base::ASCIIToUTF16("Test HTML data"), GURL());
+  data.SetHtml(u"Test HTML data", GURL());
   data_offer.SetDropData(&data_exchange_delegate, nullptr, data);
 
   base::ScopedFD read_pipe_16;
   base::ScopedFD write_pipe_16;
   ASSERT_TRUE(base::CreatePipe(&read_pipe_16, &write_pipe_16));
   data_offer.Receive("text/html;charset=utf-16", std::move(write_pipe_16));
-  base::string16 result_16;
+  std::u16string result_16;
   ASSERT_TRUE(ReadString16(std::move(read_pipe_16), &result_16));
-  EXPECT_EQ(base::ASCIIToUTF16("Test HTML data"), result_16);
+  EXPECT_EQ(u"Test HTML data", result_16);
 
   base::ScopedFD read_pipe_8;
   base::ScopedFD write_pipe_8;
@@ -390,9 +420,30 @@ TEST_F(DataOfferTest,
   urls.push_back(GURL(""));
   data_exchange_delegate.RunSendPickleCallback(urls);
 
-  base::string16 result;
+  std::u16string result;
   ASSERT_TRUE(ReadString16(std::move(read_pipe), &result));
-  EXPECT_EQ(base::ASCIIToUTF16(""), result);
+  EXPECT_EQ(u"", result);
+}
+
+TEST_F(DataOfferTest, ReceiveFileContentsDropData) {
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  ui::OSExchangeData data;
+  const std::string expected = "test data";
+  data.provider().SetFileContents(base::FilePath("test.jpg"), expected);
+  data_offer.SetDropData(&data_exchange_delegate, nullptr, data);
+
+  base::ScopedFD read_pipe;
+  base::ScopedFD write_pipe;
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+
+  data_offer.Receive("application/octet-stream;name=\"test.jpg\"",
+                     std::move(write_pipe));
+  std::string result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &result));
+  EXPECT_EQ(expected, result);
 }
 
 TEST_F(DataOfferTest, SetClipboardDataPlainText) {
@@ -402,7 +453,7 @@ TEST_F(DataOfferTest, SetClipboardDataPlainText) {
   TestDataExchangeDelegate data_exchange_delegate;
   {
     ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
-    writer.WriteText(base::UTF8ToUTF16("Test data"));
+    writer.WriteText(u"Test data");
   }
 
   auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
@@ -434,11 +485,268 @@ TEST_F(DataOfferTest, SetClipboardDataPlainText) {
   // Read as utf-16.
   ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
   data_offer.Receive("text/plain;charset=utf-16", std::move(write_pipe));
-  base::string16 result16;
+  std::u16string result16;
   ASSERT_TRUE(ReadString16(std::move(read_pipe), &result16));
   EXPECT_EQ("Test data", base::UTF16ToUTF8(result16));
 }
 
+TEST_F(DataOfferTest, SetClipboardDataOfferDteToLacros) {
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  data_exchange_delegate.set_endpoint_type(ui::EndpointType::kLacros);
+  {
+    ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
+    writer.SetDataSource(std::make_unique<ui::DataTransferEndpoint>(
+        (GURL("https://www.google.com"))));
+    writer.WriteText(u"Test data");
+  }
+
+  auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
+  data_offer.SetClipboardData(
+      &data_exchange_delegate, *ui::Clipboard::GetForCurrentThread(),
+      data_exchange_delegate.GetDataTransferEndpointType(window));
+
+  EXPECT_EQ(4u, delegate.mime_types().size());
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-8"));
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-16"));
+  EXPECT_EQ(1u, delegate.mime_types().count("UTF8_STRING"));
+  EXPECT_EQ(1u,
+            delegate.mime_types().count("chromium/x-data-transfer-endpoint"));
+
+  base::ScopedFD read_pipe;
+  base::ScopedFD write_pipe;
+
+  // Read as utf-8.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("text/plain;charset=utf-8", std::move(write_pipe));
+  std::string text_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &text_result));
+  EXPECT_EQ("Test data", text_result);
+
+  // Retrieve encoded clipboard source data transfer endpoint.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("chromium/x-data-transfer-endpoint",
+                     std::move(write_pipe));
+  std::string dte_json_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &dte_json_result));
+  EXPECT_EQ(R"({"endpoint_type":"url","url":"https://www.google.com/"})",
+            dte_json_result);
+}
+
+TEST_F(DataOfferTest, SetClipboardDataDoNotOfferDteToNonLacros) {
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  data_exchange_delegate.set_endpoint_type(ui::EndpointType::kArc);
+  {
+    ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
+    writer.SetDataSource(std::make_unique<ui::DataTransferEndpoint>(
+        (GURL("https://www.google.com"))));
+    writer.WriteText(u"Test data");
+  }
+
+  auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
+  data_offer.SetClipboardData(
+      &data_exchange_delegate, *ui::Clipboard::GetForCurrentThread(),
+      data_exchange_delegate.GetDataTransferEndpointType(window));
+
+  EXPECT_EQ(3u, delegate.mime_types().size());
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-8"));
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-16"));
+  EXPECT_EQ(1u, delegate.mime_types().count("UTF8_STRING"));
+  EXPECT_EQ(0u,
+            delegate.mime_types().count("chromium/x-data-transfer-endpoint"));
+
+  base::ScopedFD read_pipe;
+  base::ScopedFD write_pipe;
+
+  // Read as utf-8.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("text/plain;charset=utf-8", std::move(write_pipe));
+  std::string text_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &text_result));
+  EXPECT_EQ("Test data", text_result);
+
+  // Attempt to retrieve encoded clipboard source data transfer endpoint.
+  // Nothing should be returned.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("chromium/x-data-transfer-endpoint",
+                     std::move(write_pipe));
+  std::string dte_json_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &dte_json_result));
+  EXPECT_EQ("", dte_json_result);
+}
+
+// See crbug.com/1339344
+TEST_F(DataOfferTest, SetClipboardDataOfferDteToLacrosSourceChanged) {
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  data_exchange_delegate.set_endpoint_type(ui::EndpointType::kLacros);
+  {
+    ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
+    writer.SetDataSource(std::make_unique<ui::DataTransferEndpoint>(
+        (GURL("https://www.google.com"))));
+    writer.WriteText(u"Test data");
+  }
+
+  auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
+  data_offer.SetClipboardData(
+      &data_exchange_delegate, *ui::Clipboard::GetForCurrentThread(),
+      data_exchange_delegate.GetDataTransferEndpointType(window));
+
+  EXPECT_EQ(4u, delegate.mime_types().size());
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-8"));
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-16"));
+  EXPECT_EQ(1u, delegate.mime_types().count("UTF8_STRING"));
+  EXPECT_EQ(1u,
+            delegate.mime_types().count("chromium/x-data-transfer-endpoint"));
+
+  // Clipboard data changed
+  {
+    ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
+    writer.WriteText(u"Data changed with no source");
+  }
+
+  base::ScopedFD read_pipe;
+  base::ScopedFD write_pipe;
+
+  // Read as utf-8.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("text/plain;charset=utf-8", std::move(write_pipe));
+  std::string text_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &text_result));
+  EXPECT_EQ("Data changed with no source", text_result);
+
+  // Retrieve encoded clipboard source data transfer endpoint.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+#if DCHECK_IS_ON()
+  EXPECT_DEATH_IF_SUPPORTED(
+      data_offer.Receive("chromium/x-data-transfer-endpoint",
+                         std::move(write_pipe));
+      ,
+      "Check failed: data_src. Clipboard source DataTransferEndpoint has "
+      "changed after initial MIME advertising. If you see this please file a "
+      "bug and contact the chromeos-dlp team.");
+#else
+  data_offer.Receive("chromium/x-data-transfer-endpoint",
+                     std::move(write_pipe));
+  std::string dte_json_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &dte_json_result));
+  EXPECT_EQ("", dte_json_result);
+#endif
+}
+
+TEST_F(DataOfferTest, SetDropDataOfferDteToLacros) {
+  base::flat_set<DndAction> source_actions;
+  source_actions.insert(DndAction::kCopy);
+  source_actions.insert(DndAction::kMove);
+
+  ui::OSExchangeData data;
+  data.SetString(std::u16string(u"Test data"));
+  data.SetSource(std::make_unique<ui::DataTransferEndpoint>(
+      (GURL("https://www.google.com"))));
+
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  EXPECT_EQ(0u, delegate.mime_types().size());
+  EXPECT_EQ(0u, delegate.source_actions().size());
+  EXPECT_EQ(DndAction::kNone, delegate.dnd_action());
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  data_exchange_delegate.set_endpoint_type(ui::EndpointType::kLacros);
+
+  data_offer.SetDropData(&data_exchange_delegate, nullptr, data);
+  data_offer.SetSourceActions(source_actions);
+  data_offer.SetActions(base::flat_set<DndAction>(), DndAction::kMove);
+
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-8"));
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-16"));
+  EXPECT_EQ(1u,
+            delegate.mime_types().count("chromium/x-data-transfer-endpoint"));
+  EXPECT_EQ(2u, delegate.source_actions().size());
+  EXPECT_EQ(1u, delegate.source_actions().count(DndAction::kCopy));
+  EXPECT_EQ(1u, delegate.source_actions().count(DndAction::kMove));
+  EXPECT_EQ(DndAction::kMove, delegate.dnd_action());
+
+  base::ScopedFD read_pipe;
+  base::ScopedFD write_pipe;
+
+  // Read as utf-8.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("text/plain;charset=utf-8", std::move(write_pipe));
+  std::string text_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &text_result));
+  EXPECT_EQ("Test data", text_result);
+
+  // Retrieve encoded drag source data transfer endpoint.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("chromium/x-data-transfer-endpoint",
+                     std::move(write_pipe));
+  std::string dte_json_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &dte_json_result));
+  EXPECT_EQ(R"({"endpoint_type":"url","url":"https://www.google.com/"})",
+            dte_json_result);
+}
+
+TEST_F(DataOfferTest, SetDropDataDoNotOfferDteToNonLacros) {
+  base::flat_set<DndAction> source_actions;
+  source_actions.insert(DndAction::kCopy);
+  source_actions.insert(DndAction::kMove);
+
+  ui::OSExchangeData data;
+  data.SetString(std::u16string(u"Test data"));
+  data.SetSource(std::make_unique<ui::DataTransferEndpoint>(
+      (GURL("https://www.google.com"))));
+
+  TestDataOfferDelegate delegate;
+  DataOffer data_offer(&delegate);
+
+  EXPECT_EQ(0u, delegate.mime_types().size());
+  EXPECT_EQ(0u, delegate.source_actions().size());
+  EXPECT_EQ(DndAction::kNone, delegate.dnd_action());
+
+  TestDataExchangeDelegate data_exchange_delegate;
+  data_exchange_delegate.set_endpoint_type(ui::EndpointType::kCrostini);
+
+  data_offer.SetDropData(&data_exchange_delegate, nullptr, data);
+  data_offer.SetSourceActions(source_actions);
+  data_offer.SetActions(base::flat_set<DndAction>(), DndAction::kMove);
+
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-8"));
+  EXPECT_EQ(1u, delegate.mime_types().count("text/plain;charset=utf-16"));
+  EXPECT_EQ(0u,
+            delegate.mime_types().count("chromium/x-data-transfer-endpoint"));
+  EXPECT_EQ(2u, delegate.source_actions().size());
+  EXPECT_EQ(1u, delegate.source_actions().count(DndAction::kCopy));
+  EXPECT_EQ(1u, delegate.source_actions().count(DndAction::kMove));
+  EXPECT_EQ(DndAction::kMove, delegate.dnd_action());
+
+  base::ScopedFD read_pipe;
+  base::ScopedFD write_pipe;
+
+  // Read as utf-8.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("text/plain;charset=utf-8", std::move(write_pipe));
+  std::string text_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &text_result));
+  EXPECT_EQ("Test data", text_result);
+
+  // Attempt to retrieve encoded drag source data transfer endpoint.
+  // Nothing should be returned.
+  ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
+  data_offer.Receive("chromium/x-data-transfer-endpoint",
+                     std::move(write_pipe));
+  std::string dte_json_result;
+  ASSERT_TRUE(ReadString(std::move(read_pipe), &dte_json_result));
+  EXPECT_EQ("", dte_json_result);
+}
+
 TEST_F(DataOfferTest, SetClipboardDataHTML) {
   TestDataOfferDelegate delegate;
   DataOffer data_offer(&delegate);
@@ -446,7 +754,7 @@ TEST_F(DataOfferTest, SetClipboardDataHTML) {
   TestDataExchangeDelegate data_exchange_delegate;
   {
     ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
-    writer.WriteHTML(base::UTF8ToUTF16("Test data"), "");
+    writer.WriteHTML(u"Test data", "", ui::ClipboardContentType::kSanitized);
   }
 
   auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
@@ -469,7 +777,7 @@ TEST_F(DataOfferTest, SetClipboardDataHTML) {
 
   ASSERT_TRUE(base::CreatePipe(&read_pipe, &write_pipe));
   data_offer.Receive("text/html;charset=utf-16", std::move(write_pipe));
-  base::string16 result16;
+  std::u16string result16;
   ASSERT_TRUE(ReadString16(std::move(read_pipe), &result16));
   EXPECT_EQ("Test data", base::UTF16ToUTF8(result16));
 }
@@ -542,8 +850,7 @@ TEST_F(DataOfferTest, SetClipboardDataImage) {
   ASSERT_TRUE(gfx::PNGCodec::Decode(
       reinterpret_cast<const unsigned char*>(result.data()), result.size(),
       &decoded));
-  EXPECT_TRUE(cc::MatchesBitmap(
-      image, decoded, cc::ExactPixelComparator(/*discard_alpha=*/false)));
+  EXPECT_TRUE(cc::MatchesBitmap(image, decoded, cc::ExactPixelComparator()));
   std::string good = result;
   ASSERT_TRUE(ReadString(std::move(read_pipe2), &result));
   EXPECT_EQ(good, result);
@@ -565,7 +872,7 @@ TEST_F(DataOfferTest, SetClipboardDataFilenames) {
   {
     ui::ScopedClipboardWriter writer(ui::ClipboardBuffer::kCopyPaste);
     writer.WritePickledData(pickle,
-                            ui::ClipboardFormatType::GetWebCustomDataType());
+                            ui::ClipboardFormatType::WebCustomDataType());
   }
 
   auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
@@ -603,7 +910,7 @@ TEST_F(DataOfferTest, SetClipboardDataWithTransferPolicy) {
     ui::ScopedClipboardWriter writer(
         ui::ClipboardBuffer::kCopyPaste,
         std::make_unique<ui::DataTransferEndpoint>(ui::EndpointType::kArc));
-    writer.WriteText(base::UTF8ToUTF16("Test data"));
+    writer.WriteText(u"Test data");
   }
 
   auto* window = CreateTestWindowInShellWithBounds(gfx::Rect());
