@@ -1,4 +1,4 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,13 +9,13 @@
 #include <set>
 #include <utility>
 
-#include "base/callback.h"
-#include "base/macros.h"
-#include "base/memory/ref_counted.h"
+#include "base/functional/callback.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
-#include "build/chromeos_buildflags.h"
+#include "base/time/time.h"
 #include "cc/base/region.h"
+#include "chromeos/ui/frame/caption_buttons/snap_controller.h"
+#include "components/exo/buffer.h"
 #include "components/exo/layer_tree_frame_sink_holder.h"
 #include "components/exo/surface_delegate.h"
 #include "components/viz/common/frame_sinks/begin_frame_source.h"
@@ -23,8 +23,12 @@
 #include "third_party/skia/include/core/SkBlendMode.h"
 #include "ui/aura/window.h"
 #include "ui/gfx/geometry/rect.h"
+#include "ui/gfx/geometry/rounded_corners_f.h"
+#include "ui/gfx/geometry/rrect_f.h"
+#include "ui/gfx/geometry/size_conversions.h"
+#include "ui/gfx/geometry/size_f.h"
+#include "ui/gfx/geometry/transform.h"
 #include "ui/gfx/native_widget_types.h"
-#include "ui/gfx/transform.h"
 
 class SkPath;
 
@@ -41,7 +45,8 @@ class TracedValue;
 namespace gfx {
 class ColorSpace;
 class GpuFence;
-}
+struct PresentationFeedback;
+}  // namespace gfx
 
 namespace viz {
 class CompositorFrame;
@@ -49,6 +54,7 @@ class CompositorFrame;
 
 namespace exo {
 class Buffer;
+class SecurityDelegate;
 class FrameSinkResourceManager;
 class SurfaceObserver;
 
@@ -57,32 +63,59 @@ class PropertyHelper;
 }
 
 // Counter-clockwise rotations.
-enum class Transform { NORMAL, ROTATE_90, ROTATE_180, ROTATE_270 };
+enum class Transform {
+  NORMAL,
+  ROTATE_90,
+  ROTATE_180,
+  ROTATE_270,
+  FLIPPED,
+  FLIPPED_ROTATE_90,
+  FLIPPED_ROTATE_180,
+  FLIPPED_ROTATE_270
+};
+
+// Priority for overlay promotion.
+enum class OverlayPriority { LOW, REGULAR, REQUIRED };
 
 // A property key to store the surface Id set by the client.
 extern const ui::ClassProperty<std::string*>* const kClientSurfaceIdKey;
 
+// A property key to store the window session Id set by client or full_restore
+// component.
+extern const ui::ClassProperty<int32_t>* const kWindowSessionId;
+
 // This class represents a rectangular area that is displayed on the screen.
 // It has a location, size and pixel contents.
 class Surface final : public ui::PropertyHandler {
  public:
   using PropertyDeallocator = void (*)(int64_t value);
-  using LeaveEnterCallback = base::RepeatingCallback<void(int64_t, int64_t)>;
+  using LeaveEnterCallback = base::RepeatingCallback<bool(int64_t, int64_t)>;
 
   Surface();
+
+  Surface(const Surface&) = delete;
+  Surface& operator=(const Surface&) = delete;
+
   ~Surface() override;
 
   // Type-checking downcast routine.
   static Surface* AsSurface(const aura::Window* window);
 
-  aura::Window* window() { return window_.get(); }
+  aura::Window* window() const { return window_.get(); }
 
   void set_leave_enter_callback(LeaveEnterCallback callback) {
     leave_enter_callback_ = callback;
   }
 
+  void set_legacy_buffer_release_skippable(bool skippable) {
+    legacy_buffer_release_skippable_ = skippable;
+  }
+
   // Called when the display the surface is on has changed.
-  void UpdateDisplay(int64_t old_id, int64_t new_id);
+  // Returns true if successful, and false if it fails.
+  bool UpdateDisplay(int64_t old_id, int64_t new_id);
+
+  display::Display GetDisplay() const;
 
   // Called when the output is added for new display.
   void OnNewOutputAdded();
@@ -144,13 +177,27 @@ class Surface final : public ui::PropertyHandler {
   // double-buffered and will be applied when Commit() is called.
   void AddSubSurface(Surface* sub_surface);
   void RemoveSubSurface(Surface* sub_surface);
-  void SetSubSurfacePosition(Surface* sub_surface, const gfx::Point& position);
+  // Allow for finer granularity for sub surface positioning.
+  void SetSubSurfacePosition(Surface* sub_surface, const gfx::PointF& position);
   void PlaceSubSurfaceAbove(Surface* sub_surface, Surface* reference);
   void PlaceSubSurfaceBelow(Surface* sub_surface, Surface* sibling);
   void OnSubSurfaceCommit();
 
+  void SetRoundedCorners(const gfx::RRectF& rounded_corners_bounds);
+  void SetOverlayPriorityHint(OverlayPriority hint);
+
+  // Sets the surface's clip rectangle.
+  void SetClipRect(const absl::optional<gfx::RectF>& clip_rect);
+
+  // Sets the surface's transformation matrix.
+  void SetSurfaceTransform(const gfx::Transform& transform);
+
+  // Sets the background color that shall be associated with the next buffer
+  // commit.
+  void SetBackgroundColor(absl::optional<SkColor4f> background_color);
+
   // This sets the surface viewport for scaling.
-  void SetViewport(const gfx::Size& viewport);
+  void SetViewport(const gfx::SizeF& viewport);
 
   // This sets the surface crop rectangle.
   void SetCrop(const gfx::RectF& crop);
@@ -180,21 +227,27 @@ class Surface final : public ui::PropertyHandler {
   // Request that surface should have a specific application ID string.
   void SetApplicationId(const char* application_id);
 
-  // Whether to hide the shelf when fullscreen. If true, shelf is inaccessible
-  // (plain fullscreen). If false, shelf auto-hides and can be shown with a
-  // mouse gesture (immersive fullscreen).
+  // Whether to show/hide the shelf when fullscreen. If true, the titlebar/shelf
+  // will show when the mouse moves to the top/bottom of the screen. If false
+  // (plain fullscreen), the titlebar and shelf are always hidden.
   void SetUseImmersiveForFullscreen(bool value);
 
-  // Called to show the snap preview to the right or left, or to hide it.
-  void ShowSnapPreviewToRight();
-  void ShowSnapPreviewToLeft();
+  // Called to show the snap preview to the primary or secondary position, or
+  // to hide it.
+  void ShowSnapPreviewToSecondary();
+  void ShowSnapPreviewToPrimary();
   void HideSnapPreview();
 
-  // Called when the client was snapped to right or left, or reset.
-  void SetSnappedToRight();
-  void SetSnappedToLeft();
+  // Called when the client was snapped to primary or secondary position, or
+  // reset.
+  void SetSnapPrimary(float snap_ratio);
+  void SetSnapSecondary(float snap_ratio);
   void UnsetSnap();
 
+  // Whether the current client window can go back, as per its navigation list.
+  void SetCanGoBack();
+  void UnsetCanGoBack();
+
   // This sets the color space for the buffer for this surface.
   void SetColorSpace(gfx::ColorSpace color_space);
 
@@ -205,6 +258,12 @@ class Surface final : public ui::PropertyHandler {
   void SetClientSurfaceId(const char* client_surface_id);
   std::string GetClientSurfaceId() const;
 
+  // Sets whether the surface contains video.
+  void SetContainsVideo(bool contains_video);
+
+  // Returns whether this surface or any of its subsurfaces contains a video.
+  bool ContainsVideo();
+
   // Enable embedding of an arbitrary viz surface in this exo surface.
   // If the callback is valid, a SurfaceDrawQuad will be emitted targeting
   // the returned SurfaceId each frame.
@@ -220,6 +279,13 @@ class Surface final : public ui::PropertyHandler {
   // Returns whether the surface has an uncommitted acquire fence.
   bool HasPendingAcquireFence() const;
 
+  // Request a callback when the buffer attached at the next commit is
+  // no longer used by that commit.
+  void SetPerCommitBufferReleaseCallback(
+      Buffer::PerCommitExplicitReleaseCallback callback);
+  // Whether the surface has an uncommitted per-commit buffer release callback.
+  bool HasPendingPerCommitBufferReleaseCallback() const;
+
   // Surface state (damage regions, attached buffers, etc.) is double-buffered.
   // A Commit() call atomically applies all pending state, replacing the
   // current state. Commit() is not guaranteed to be synchronous. See
@@ -240,8 +306,9 @@ class Surface final : public ui::PropertyHandler {
 
   // This will append contents for surface and its descendants to frame.
   void AppendSurfaceHierarchyContentsToFrame(
-      const gfx::Point& origin,
+      const gfx::PointF& origin,
       float device_scale_factor,
+      bool client_submits_in_pixel_coords,
       FrameSinkResourceManager* resource_manager,
       viz::CompositorFrame* frame);
 
@@ -279,7 +346,7 @@ class Surface final : public ui::PropertyHandler {
   void SetBeginFrameSource(viz::BeginFrameSource* begin_frame_source);
 
   // Returns the active content size.
-  const gfx::Size& content_size() const { return content_size_; }
+  const gfx::SizeF& content_size() const { return content_size_; }
 
   // Returns the active content bounds for surface hierarchy. ie. the bounding
   // box of the surface and its descendants, in the local coordinate space of
@@ -310,6 +377,11 @@ class Surface final : public ui::PropertyHandler {
   // Triggers sending an occlusion update to observers.
   void OnWindowOcclusionChanged();
 
+  // Triggers sending a locking status to observers.
+  // true : lock a frame to normal or restore state
+  // false : unlock the previously locked frame
+  void SetFrameLocked(bool lock);
+
   // True if the window for this surface has its occlusion tracked.
   bool IsTrackingOcclusion();
 
@@ -319,6 +391,65 @@ class Surface final : public ui::PropertyHandler {
   // Requests that this surface should be made active (i.e. foregrounded).
   void RequestActivation();
 
+  // Requests that surface my have a window session ID assigned by client or
+  // full_restore component.
+  void SetWindowSessionId(int32_t window_session_id);
+  int32_t GetWindowSessionId();
+
+  // Requests that the surface enters PIP mode.
+  void SetPip();
+
+  // Requests that the surface exits PIP mode.
+  void UnsetPip();
+
+  // Requests that the surface maintains the given aspect ratio.
+  void SetAspectRatio(const gfx::SizeF& aspect_ratio);
+
+  // Triggers send desk state of the window to observers.
+  // |state| is the index of the desk which the window moved to,
+  // or -1 for a window assigned to all desks.
+  void OnDeskChanged(int state);
+
+  // Requests that DesksController to move the window to a desk at |desk_index|.
+  void MoveToDesk(int desk_index);
+
+  // Requests that window is visible on all workspaces.
+  void SetVisibleOnAllWorkspaces();
+
+  // Sets the initial workspace to restore a window to the corresponding desk.
+  void SetInitialWorkspace(const char* initial_workspace);
+
+  // Pins/locks a window to the screen so that the user cannot do anything
+  // else before the mode is released. If trusted is set, it is an invocation
+  // from a trusted app like a school test mode app.
+  void Pin(bool trusted);
+
+  // Release the pinned mode and allows the user to do other things again.
+  void Unpin();
+
+  // Starts or ends throttling on the surface.
+  void ThrottleFrameRate(bool on);
+
+  // Informs tooltip is shown.
+  void OnTooltipShown(const std::u16string& text, const gfx::Rect& bounds);
+
+  // Informs tooltip is hidden.
+  void OnTooltipHidden();
+
+  // If true is set, if this window has a focus, key events should be sent to
+  // the app, even if it is an ash shortcut (with some exceptions).
+  // See exo::Keyboard for more details.
+  void SetKeyboardShortcutsInhibited(bool inhibited);
+
+  // Returns whether keyboard shortcuts are inhibited.
+  bool is_keyboard_shortcuts_inhibited() const {
+    return keyboard_shortcuts_inhibited_;
+  }
+
+  // Returns the SecurityDelegate associated with this surface, or nullptr
+  // if one can not be determined. See go/secure-exo-ids for more details.
+  SecurityDelegate* GetSecurityDelegate();
+
  private:
   struct State {
     State();
@@ -328,24 +459,33 @@ class Surface final : public ui::PropertyHandler {
     bool operator!=(const State& other) const { return !(*this == other); }
 
     cc::Region opaque_region;
-    base::Optional<cc::Region> input_region;
+    absl::optional<cc::Region> input_region;
     int input_outset = 0;
     float buffer_scale = 1.0f;
     Transform buffer_transform = Transform::NORMAL;
-    gfx::Size viewport;
+    gfx::SizeF viewport;
     gfx::RectF crop;
     bool only_visible_on_secure_output = false;
     SkBlendMode blend_mode = SkBlendMode::kSrcOver;
     float alpha = 1.0f;
     gfx::Vector2d offset;
-    gfx::ColorSpace color_space;
+    gfx::ColorSpace color_space = gfx::ColorSpace::CreateSRGB();
     bool is_tracking_occlusion = false;
+    // Represents optional background color that must be associated with the
+    // next buffer commit.
+    absl::optional<SkColor4f> background_color;
+    bool contains_video = false;
   };
   class BufferAttachment {
    public:
     BufferAttachment();
+
+    BufferAttachment(const BufferAttachment&) = delete;
+    BufferAttachment& operator=(const BufferAttachment&) = delete;
+
     ~BufferAttachment();
 
+    BufferAttachment(BufferAttachment&& buffer);
     BufferAttachment& operator=(BufferAttachment&& buffer);
 
     base::WeakPtr<Buffer>& buffer();
@@ -356,10 +496,25 @@ class Surface final : public ui::PropertyHandler {
    private:
     base::WeakPtr<Buffer> buffer_;
     gfx::Size size_;
-
-    DISALLOW_COPY_AND_ASSIGN(BufferAttachment);
   };
 
+  // State for this surface. State is committed in a three step process:
+  // 1. Pending state is accummulated into before commit.
+  // 2. On commit, state is copied to a cached state. This is to support
+  //    synchronized commit of a tree of surfaces. When the tree of surfaces is
+  //    set to be synchronized, the state of the tree will not be committed
+  //    until the root of the tree (precisely, until a unsynchronized root of a
+  //    subtree) is committed.
+  // 3. State is committed.
+  // Some fields are persisted between commits (e.g. which buffer is attached),
+  // and some fields are not (e.g. acquire fence). For fields that are
+  // persisted, they either need to be copyable, or if they are move only, they
+  // need to be wrapped in absl::optional and only copied on commit if they
+  // have been changed. Not doing this can lead to broken behaviour, such as
+  // losing the attached buffer if some unrelated field is updated in a commit.
+  // If you add new fields to this struct, please document whether the field
+  // should be persisted between commits.
+  // See crbug.com/1283305 for context.
   struct ExtendedState {
     ExtendedState();
     ~ExtendedState();
@@ -367,17 +522,42 @@ class Surface final : public ui::PropertyHandler {
     State basic_state;
 
     // The buffer that will become the content of surface.
-    BufferAttachment buffer;
+    // Persisted between commits.
+    absl::optional<BufferAttachment> buffer;
+    // The rounded corners bounds for the surface.
+    // Persisted between commits.
+    gfx::RRectF rounded_corners_bounds;
     // The damage region to schedule paint for.
+    // Not persisted between commits.
     cc::Region damage;
     // These lists contain the callbacks to notify the client when it is a good
     // time to start producing a new frame.
+    // Not persisted between commits.
     std::list<FrameCallback> frame_callbacks;
     // These lists contain the callbacks to notify the client when surface
     // contents have been presented.
+    // Not persisted between commits.
     std::list<PresentationCallback> presentation_callbacks;
     // The acquire gpu fence to associate with the surface buffer.
+    // Not persisted between commits.
     std::unique_ptr<gfx::GpuFence> acquire_fence;
+    // Callback to notify about the per-commit buffer release. The wayland
+    // Exo backend uses this callback to implement the immediate_release
+    // event of the explicit sync protocol.
+    // Not persisted between commits.
+    Buffer::PerCommitExplicitReleaseCallback
+        per_commit_explicit_release_callback_;
+    // The hint for overlay prioritization
+    // Persisted between commits.
+    OverlayPriority overlay_priority_hint = OverlayPriority::REGULAR;
+    // The clip rect for this surface, in the parent's coordinate space. This
+    // should only be set for subsurfaces.
+    // Persisted between commits.
+    absl::optional<gfx::RectF> clip_rect;
+    // The transform to apply when drawing this surface. This should only be set
+    // for subsurfaces, and doesn't apply to children of this surface.
+    // Persisted between commits.
+    gfx::Transform surface_transform;
   };
 
   friend class subtle::PropertyHelper;
@@ -393,15 +573,18 @@ class Surface final : public ui::PropertyHandler {
 
   // Puts the current surface into a draw quad, and appends the draw quads into
   // the |frame|.
-  void AppendContentsToFrame(const gfx::Point& origin,
+  void AppendContentsToFrame(const gfx::PointF& origin,
                              float device_scale_factor,
+                             bool client_submits_in_pixel_coords,
                              viz::CompositorFrame* frame);
 
   // Update surface content size base on current buffer size.
   void UpdateContentSize();
 
   // This returns true when the surface has some contents assigned to it.
-  bool has_contents() const { return !state_.buffer.size().IsEmpty(); }
+  bool has_contents() const {
+    return state_.buffer.has_value() && !state_.buffer->size().IsEmpty();
+  }
 
   // This window has the layer which contains the Surface contents.
   std::unique_ptr<aura::Window> window_;
@@ -410,7 +593,7 @@ class Surface final : public ui::PropertyHandler {
   bool sub_surfaces_changed_ = false;
 
   // This is the size of the last committed contents.
-  gfx::Size content_size_;
+  gfx::SizeF content_size_;
 
   // This is the bounds of the last committed surface hierarchy contents.
   gfx::Rect surface_hierarchy_content_bounds_;
@@ -435,7 +618,7 @@ class Surface final : public ui::PropertyHandler {
   // The stack of sub-surfaces to take effect when Commit() is called.
   // Bottom-most sub-surface at the front of the list and top-most sub-surface
   // at the back.
-  using SubSurfaceEntry = std::pair<Surface*, gfx::Point>;
+  using SubSurfaceEntry = std::pair<Surface*, gfx::PointF>;
   using SubSurfaceEntryList = std::list<SubSurfaceEntry>;
   SubSurfaceEntryList pending_sub_surfaces_;
   SubSurfaceEntryList sub_surfaces_;
@@ -469,9 +652,7 @@ class Surface final : public ui::PropertyHandler {
   // Surface observer list. Surface does not own the observers.
   base::ObserverList<SurfaceObserver, true>::Unchecked observers_;
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
   std::unique_ptr<ash::OutputProtectionDelegate> output_protection_;
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
   viz::SurfaceId first_embedded_surface_id_;
   viz::SurfaceId latest_embedded_surface_id_;
@@ -483,20 +664,23 @@ class Surface final : public ui::PropertyHandler {
 
   LeaveEnterCallback leave_enter_callback_;
 
-  DISALLOW_COPY_AND_ASSIGN(Surface);
+  bool keyboard_shortcuts_inhibited_ = false;
+  bool legacy_buffer_release_skippable_ = false;
 };
 
 class ScopedSurface {
  public:
   ScopedSurface(Surface* surface, SurfaceObserver* observer);
+
+  ScopedSurface(const ScopedSurface&) = delete;
+  ScopedSurface& operator=(const ScopedSurface&) = delete;
+
   virtual ~ScopedSurface();
   Surface* get() { return surface_; }
 
  private:
   Surface* const surface_;
   SurfaceObserver* const observer_;
-
-  DISALLOW_COPY_AND_ASSIGN(ScopedSurface);
 };
 
 }  // namespace exo
